using System;
using System.Collections.Generic;
using System.Text;

using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Net;
using System.Net.Mail;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Threading;
using System.Windows.Forms;

namespace CCPM
{
	class ExceptionHandler
	{
		#region Entry Points
		public static void AddHandler( bool screenshot, bool showUser, bool kill )
		{
			_makeScreenshot = screenshot;
			_logToUI = showUser;
			_killAppOnException = kill;

			// attempt to load optional settings from .config file
			//LoadConfigSettings();

			// track the parent assembly that set up error handling
			// need to call this NOW so we set it appropriately; otherwise
			// we may get the wrong assembly at exception time!
			if( Assembly.GetEntryAssembly() == null )
				_parentAssembly = Assembly.GetCallingAssembly();
			else
				_parentAssembly = Assembly.GetEntryAssembly();

			// Attach handler
			Application.ThreadException += new ThreadExceptionEventHandler( ThreadExceptionHandler );

			// AppDomain.UnhandledException doesn't give the explicit catches a chance,
			//  but I'm leaving it here, commented out, for clarity.
			//System.AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(DomainExceptionHandler);
		}

		private static void DomainExceptionHandler( object sender, UnhandledExceptionEventArgs e )
		{
			HandleException( (Exception)e.ExceptionObject );

			if( _killAppOnException )
			{
				Application.Exit();
			}
		}

		private static void ThreadExceptionHandler( object sender, System.Threading.ThreadExceptionEventArgs args )
		{
			HandleException( args.Exception );

			if( _killAppOnException )
			{
				Application.Exit();
			}
		}

		public static void HandleException( Exception exceptn )
		{
			HandleException( string.Empty, exceptn );
		}

		public static void HandleException( string msg, Exception exceptn )
		{
			HandleException( msg, exceptn, _logToUI );
		}

		public static void HandleException( string msg, Exception exceptn, bool showUI )
		{
			Cursor.Current = System.Windows.Forms.Cursors.WaitCursor;

			// Initialize properties
			_exceptionText = msg;

			if( !string.IsNullOrEmpty( _exceptionText ) )
			{
				_exceptionText += Environment.NewLine;
			}

			_exceptionType = string.Empty;

			_screenshotFullName = string.Empty;
			_logfileFullName = string.Empty;

			_logToScreenshotOK = false;
			_logToDatabaseOK = false;
			_logToFileOK = false;
			_logToEventLogOK = false;

			// turn the exception into an informative string
			try
			{
				_exceptionText += ExceptionToString( exceptn );
				_exceptionType = exceptn.GetType().FullName;
			}
			catch( Exception ex )
			{
				_exceptionText += "Error while generating exception string: " + ex.ToString();
				_exceptionType = ex.GetType().FullName;
			}

			if( _makeScreenshot )
			{
				try
				{
					TakeScreenshot();
					_exceptionText += Environment.NewLine + "Screenshot placed at:	" + _screenshotFullName;
				}
				catch( Exception ex )
				{
					_exceptionText += Environment.NewLine + "Screenshot Error:" + ExceptionToString( ex );
				}
			}

			// Make attempts to log the error, using backup plans if necessary
			// Attempts are made in order of preference
			try
			{
				ExceptionToDatabase();
			}
			catch( Exception ex1 )
			{
				_exceptionText += Environment.NewLine + "Database Error:" + ExceptionToString( ex1 );

				try
				{
					ExceptionToEventLog();
				}
				catch( Exception ex2 )
				{
					_exceptionText += Environment.NewLine + "Event Log Error:" + ExceptionToString( ex2 );

					try
					{
						ExceptionToFile();
					}
					catch
					{
						// Just eat it, nothing's working  >.<
					}
				}
			}

			// email takes under 1 second
			ExceptionToEmail();

			Cursor.Current = System.Windows.Forms.Cursors.Default;

			// display message to the user
			if( showUI )
			{
				try
				{
					ExceptionToUI();
				}
				catch
				{
					// Already logged other problems, can't attempt to log this for the risk of a closed loop
				}
			}
		}
		#endregion

		#region Config Fields
		private static bool _makeScreenshot = true;
		private static bool _logToUI = true;
		private static bool _killAppOnException = true;

		private static string _screenshotFileName = "VtmError";
		private static string _logfileName = "Error.log";
		private static ImageFormat _screenshotImageFormat = ImageFormat.Png;

		private static Assembly _parentAssembly = null;
		#endregion

		#region Exception Processing State Fields
		private static string _exceptionText = string.Empty;
		private static string _exceptionType = string.Empty;

		private static string _screenshotFullName = string.Empty;
		private static string _logfileFullName = string.Empty;

		private static bool _logToScreenshotOK = false;
		private static bool _logToDatabaseOK = false;
		private static bool _logToFileOK = false;
		private static bool _logToEventLogOK = false;
		#endregion

		#region Win32api screenshot calls
		// Windows API calls necessary to support screen capture
		[DllImport( "gdi32.dll" )]
		private static extern int BitBlt( int hDestDC, int x, int y, int nWidth, int nHeight, int hSrcDC, int xSrc, int ySrc, int dwRop );

		[DllImport( "user32.dll" )]
		private static extern int GetDC( int hwnd );

		[DllImport( "user32.dll" )]
		private static extern int ReleaseDC( int hwnd, int hdc );
		#endregion

		#region Inputs
		private static string ApplicationPath
		{
			get { return AppDomain.CurrentDomain.BaseDirectory; }
		}

		private static void TakeScreenshot()
		{
			// note that screenshotname does not include the file type extension
			try
			{
				Rectangle screenRect = Screen.PrimaryScreen.Bounds;
				Bitmap screenBitmap = new Bitmap( screenRect.Width, screenRect.Height );
				Graphics screenGraphics = Graphics.FromImage( screenBitmap );

				// get a device context to the windows desktop and our destination  bitmaps
				int hdcSrc = GetDC( 0 );
				IntPtr hdcDest = screenGraphics.GetHdc();

				// copy what is on the desktop to the bitmap
				const int SRCCOPY = 0xCC0020;
				BitBlt( hdcDest.ToInt32(), 0, 0, screenRect.Width, screenRect.Height, hdcSrc, 0, 0, SRCCOPY );

				// release device contexts
				screenGraphics.ReleaseHdc( hdcDest );
				ReleaseDC( 0, hdcSrc );

				string extension = "." + _screenshotImageFormat.ToString().ToLower();

				// Find a name that isn't used (decorated by time)
				do
				{
					_screenshotFullName = Path.Combine( ApplicationPath, string.Format( "{0}[{1}]", _screenshotFileName, DateTime.Now.Ticks ) );

					if( Path.GetExtension( _screenshotFullName ) != extension )
						_screenshotFullName += extension;

				} while( File.Exists( _screenshotFullName ) );

				//switch( strFormatExtension )
				//{
				//case "jpeg":
				//   BitmapToJPEG( screenBitmap, strFilename, 80 );
				//   break;

				//default:
				screenBitmap.Save( _screenshotFullName, _screenshotImageFormat );
				//   break;
				//}

				_logToScreenshotOK = true;
			}
			catch( Exception )
			{
				_logToScreenshotOK = false;
			}
		}
		#endregion

		#region Outputs
		private static void ExceptionToUI()
		{
			string msg = string.Format( "A {0} error occurred.  Your last action probably failed to finish.", _exceptionType );

			if( _logToScreenshotOK )
				msg += Environment.NewLine + Environment.NewLine + "A screen shot of the error condition was made.";

			if( _logToDatabaseOK || _logToEventLogOK || _logToFileOK )
				msg += Environment.NewLine + Environment.NewLine + "The error has been logged.";

			if( _killAppOnException )
				msg += Environment.NewLine + Environment.NewLine + "This application may exit.  If it does, you can restart it and try your task again.";

			MessageBox.Show( msg, "Application Error" );
		}

		private static void ExceptionToFile()
		{
			try
			{
				_logfileFullName = ApplicationPath + _logfileName;
				using( StreamWriter writer = new StreamWriter( _logfileFullName, true ) )
				{
					writer.Write( _exceptionType );
					writer.WriteLine();
					writer.Write( _exceptionText );
					writer.WriteLine();
					writer.Close();
				}
				_logToFileOK = true;
			}
			catch( Exception )
			{
				_logToFileOK = false;
			}
		}

		private static void ExceptionToEventLog()
		{
			try
			{
				EventLog.WriteEntry( AppDomain.CurrentDomain.FriendlyName, Environment.NewLine + _exceptionText, EventLogEntryType.Error );

				_logToEventLogOK = true;
			}
			catch( Exception )
			{
				_logToEventLogOK = false;
			}
		}

		private static void ExceptionToDatabase()
		{
			string connectionString = CCPM.Data.DbConnection.GetDbConnection().DbConnectionString;
			Microsoft.ApplicationBlocks.Data.SqlHelper.ExecuteNonQuery( connectionString, "ErrorLogInsert", _exceptionType, _exceptionText );
		}

		private static void ExceptionToEmail()
		{
			try
			{
				string host = System.Configuration.ConfigurationManager.AppSettings["SmtpClientHost"];
				string portString = System.Configuration.ConfigurationManager.AppSettings["SmtpClientPort"];
				string userName = System.Configuration.ConfigurationManager.AppSettings["SmtpClientUser"];
				string password = System.Configuration.ConfigurationManager.AppSettings["SmtpClientPwd"];
				string domain = System.Configuration.ConfigurationManager.AppSettings["SmtpClientDomain"];
				bool enableSsl = Boolean.Parse( System.Configuration.ConfigurationManager.AppSettings["SmtpClientUseSsl"] );

				string from = System.Configuration.ConfigurationManager.AppSettings["SmtpFromAddress"];
				string to = System.Configuration.ConfigurationManager.AppSettings["SmtpToAddress"];

				using( MailMessage msg = new MailMessage() )
				{
					msg.From = new MailAddress( from );
					msg.Subject = "ERROR: " + _exceptionType;
					msg.Body = _exceptionText;
					msg.BodyEncoding = System.Text.Encoding.UTF8;
					msg.IsBodyHtml = false;

					// Send to (optionally) multiple recipients, separated by a |
					string[] recipients = to.Split( '|', ',' );

					foreach( string recipient in recipients )
					{
						if( !string.IsNullOrEmpty( recipient.Trim() ) )
						{
							msg.To.Add( recipient.Trim() );
						}
					}

					// Only send if there are one or more recipients
					if( msg.To.Count > 0 )
					{
						SmtpClient smtpClient = new SmtpClient();
						NetworkCredential auth = CredentialCache.DefaultNetworkCredentials;

						if( !string.IsNullOrEmpty( userName ) )
						{
							if( string.IsNullOrEmpty( domain ) )
								auth = new NetworkCredential( userName, password );
							else
								auth = new NetworkCredential( userName, password, domain );
						}

						if( !string.IsNullOrEmpty( portString ) )
						{
							smtpClient.Port = int.Parse( portString );
						}

						smtpClient.EnableSsl = enableSsl;
						smtpClient.Host = host;
						smtpClient.Credentials = auth;
						smtpClient.DeliveryMethod = SmtpDeliveryMethod.Network;

						if( !string.IsNullOrEmpty( _screenshotFullName.Trim() ) )
						{
							msg.Attachments.Add( new System.Net.Mail.Attachment( _screenshotFullName ) );
						}

						smtpClient.Send( msg );
					}
				}
			}
			catch( Exception ex )
			{
				_exceptionText += Environment.NewLine + "Mail Send ERROR: " + ex.ToString();
			}
		}
		#endregion

		#region String Conversions
		/// <summary>
		/// translate exception object to string, with additional system info
		/// </summary>
		/// <param name="exceptn">Exception to be converted</param>
		/// <returns></returns>
		private static string ExceptionToString( Exception exceptn )
		{
			StringBuilder builder = new StringBuilder();

			// Inner exceptions are handled recursively
			if( !( exceptn.InnerException == null ) )
			{
				// sometimes the original exception is wrapped in a more relevant outer exception
				// the detail exception is the "inner" exception
				// see http://msdn.microsoft.com/library/default.asp?url=/library/en-us/dnbda/html/exceptdotnet.asp
				builder.Append( "(Inner Exception)" );
				builder.Append( Environment.NewLine );
				builder.Append( ExceptionToString( exceptn.InnerException ) );
				builder.Append( Environment.NewLine );
				builder.Append( "(Outer Exception)" );
				builder.Append( Environment.NewLine );
			}
			// get general system and app information
			builder.Append( SysInfoToString() );

			// get exception-specific information
			builder.Append( "Exception Source:      " );
			try
			{
				builder.Append( exceptn.Source );
			}
			catch( Exception e )
			{
				builder.Append( e.Message );
			}

			builder.Append( Environment.NewLine );

			builder.Append( "Exception Type:        " );
			try
			{
				builder.Append( exceptn.GetType().FullName );
			}
			catch( Exception e )
			{
				builder.Append( e.Message );
			}

			builder.Append( Environment.NewLine );
			builder.Append( "Exception Message:     " );

			try
			{
				builder.Append( exceptn.Message );
			}
			catch( Exception e )
			{
				builder.Append( e.Message );
			}

			builder.Append( Environment.NewLine );
			builder.Append( "Exception Target Site: " );

			try
			{
				builder.Append( exceptn.TargetSite.Name );
			}
			catch( Exception e )
			{
				builder.Append( e.Message );
			}

			builder.Append( Environment.NewLine );

			// Check for specific exception types (and output specific details if known)
			if( exceptn is SqlException )
			{
				SqlExceptionToString( builder, exceptn );
			}

			try
			{
				string x = EnhancedStackTrace( exceptn );
				builder.Append( x );
			}
			catch( Exception e )
			{
				builder.Append( e.Message );
			}

			builder.Append( Environment.NewLine );

			return builder.ToString();
		}

		private static void SqlExceptionToString( StringBuilder builder, Exception exceptn )
		{
			SqlException se = exceptn as SqlException;

			builder.Append( Environment.NewLine );
			builder.Append( "SQL Procedure:         " );
			try
			{
				builder.Append( se.Procedure );
			}
			catch( Exception e )
			{
				builder.Append( e.Message );
			}
			builder.Append( Environment.NewLine );

			builder.Append( "SQL Server:            " );
			try
			{
				builder.Append( se.Server );
			}
			catch( Exception e )
			{
				builder.Append( e.Message );
			}
			builder.Append( Environment.NewLine );

			builder.Append( "SQL State:             " );
			try
			{
				builder.Append( se.State );
			}
			catch( Exception e )
			{
				builder.Append( e.Message );
			}
			builder.Append( Environment.NewLine );

			builder.Append( "SQL Source:            " );
			try
			{
				builder.Append( se.Source );
			}
			catch( Exception e )
			{
				builder.Append( e.Message );
			}
			builder.Append( Environment.NewLine );

			builder.Append( "SQL Error Number:      " );
			try
			{
				builder.Append( se.Number );
			}
			catch( Exception e )
			{
				builder.Append( e.Message );
			}
			builder.Append( Environment.NewLine );

			builder.Append( "SQL Line Number:       " );
			try
			{
				builder.Append( se.LineNumber );
			}
			catch( Exception e )
			{
				builder.Append( e.Message );
			}
			builder.Append( Environment.NewLine );

			foreach( SqlError error in se.Errors )
			{
				builder.Append( "SQL Error Class:       " );
				try
				{
					builder.Append( error.Class );
				}
				catch( Exception e )
				{
					builder.Append( e.Message );
				}
				builder.Append( Environment.NewLine );

				builder.Append( "SQL Error Line Number: " );
				try
				{
					builder.Append( error.LineNumber );
				}
				catch( Exception e )
				{
					builder.Append( e.Message );
				}
				builder.Append( Environment.NewLine );

				builder.Append( "SQL Error Message:     " );
				try
				{
					builder.Append( error.Message );
				}
				catch( Exception e )
				{
					builder.Append( e.Message );
				}
				builder.Append( Environment.NewLine );

				builder.Append( "SQL Error Number:      " );
				try
				{
					builder.Append( error.Number );
				}
				catch( Exception e )
				{
					builder.Append( e.Message );
				}
				builder.Append( Environment.NewLine );

				builder.Append( "SQL Error Procedure:   " );
				try
				{
					builder.Append( error.Procedure );
				}
				catch( Exception e )
				{
					builder.Append( e.Message );
				}
				builder.Append( Environment.NewLine );

				builder.Append( "SQL Error Server:      " );
				try
				{
					builder.Append( error.Server );
				}
				catch( Exception e )
				{
					builder.Append( e.Message );
				}
				builder.Append( Environment.NewLine );

				builder.Append( "SQL Error Source:      " );
				try
				{
					builder.Append( error.Source );
				}
				catch( Exception e )
				{
					builder.Append( e.Message );
				}
				builder.Append( Environment.NewLine );

				builder.Append( "SQL Error State:       " );
				try
				{
					builder.Append( error.State );
				}
				catch( Exception e )
				{
					builder.Append( e.Message );
				}
				builder.Append( Environment.NewLine );
				builder.Append( Environment.NewLine );
			}
		}

		/// <summary>
		/// gather some system information that is helpful to diagnosing exception
		/// </summary>
		/// <returns></returns>
		private static string SysInfoToString()
		{
			return SysInfoToString( false );
		}

		/// <summary>
		/// gather some system information that is helpful to diagnosing exception
		/// </summary>
		/// <param name="blnIncludeStackTrace"></param>
		/// <returns></returns>
		private static string SysInfoToString( bool blnIncludeStackTrace )
		{
			StringBuilder builder = new StringBuilder();

			builder.Append( "Date and Time:         " );
			builder.Append( DateTime.Now );
			builder.Append( Environment.NewLine );

			builder.Append( "Machine Name:          " );
			try
			{
				builder.Append( Environment.MachineName );
			}
			catch( Exception e )
			{
				builder.Append( e.Message );
			}
			builder.Append( Environment.NewLine );

			builder.Append( "IP Address:            " );
			builder.Append( GetCurrentIP() );
			builder.Append( Environment.NewLine );

			builder.Append( "Current User:          " );
			builder.Append( GetUserIdentity() );
			builder.Append( Environment.NewLine );
			builder.Append( Environment.NewLine );

			builder.Append( "Application Domain:    " );
			try
			{
				builder.Append( System.AppDomain.CurrentDomain.FriendlyName );
			}
			catch( Exception e )
			{
				builder.Append( e.Message );
			}


			builder.Append( Environment.NewLine );
			builder.Append( "Assembly Codebase:     " );
			try
			{
				builder.Append( _parentAssembly.CodeBase );
			}
			catch( Exception e )
			{
				builder.Append( e.Message );
			}
			builder.Append( Environment.NewLine );

			builder.Append( "Assembly Full Name:    " );
			try
			{
				builder.Append( _parentAssembly.FullName );
			}
			catch( Exception e )
			{
				builder.Append( e.Message );
			}
			builder.Append( Environment.NewLine );

			builder.Append( "Assembly Version:      " );
			try
			{
				builder.Append( _parentAssembly.GetName().Version.ToString() );
			}
			catch( Exception e )
			{
				builder.Append( e.Message );
			}
			builder.Append( Environment.NewLine );

			builder.Append( "Assembly Build Date:   " );
			try
			{
				builder.Append( GetAssemblyBuildDate( _parentAssembly, false ).ToString() );
			}
			catch( Exception e )
			{
				builder.Append( e.Message );
			}
			builder.Append( Environment.NewLine );
			builder.Append( Environment.NewLine );

			if( blnIncludeStackTrace )
				builder.Append( EnhancedStackTrace() );

			return builder.ToString();
		}

		#region Enhanced Stack Trace
		/// <summary>
		/// enhanced stack trace generator
		/// </summary>
		/// <param name="objStackTrace"></param>
		/// <returns></returns>
		private static string EnhancedStackTrace( StackTrace objStackTrace )
		{
			return EnhancedStackTrace( objStackTrace, string.Empty );
		}
		/// <summary>
		/// enhanced stack trace generator (exception)
		/// </summary>
		/// <param name="ex"></param>
		/// <returns></returns>
		private static string EnhancedStackTrace( Exception ex )
		{
			StackTrace trace = new StackTrace( ex );
			return EnhancedStackTrace( trace );
		}
		/// <summary>
		/// enhanced stack trace generator (no params)
		/// </summary>
		/// <returns></returns>
		private static string EnhancedStackTrace()
		{
			StackTrace trace = new StackTrace( true );
			return EnhancedStackTrace( trace, typeof( ExceptionHandler ).Name );
		}
		/// <summary>
		/// enhanced stack trace generator (prebuilt stack trace, class name to skip)
		/// </summary>
		/// <param name="objStackTrace"></param>
		/// <param name="strSkipClassName"></param>
		/// <returns></returns>
		private static string EnhancedStackTrace( StackTrace objStackTrace, string strSkipClassName )
		{
			System.Text.StringBuilder sb = new StringBuilder();

			sb.Append( Environment.NewLine );
			sb.Append( "---- Stack Trace ----" );
			sb.Append( Environment.NewLine );

			for( int intFrame = 0; intFrame < objStackTrace.FrameCount; intFrame++ )
			{
				StackFrame sf = objStackTrace.GetFrame( intFrame );
				MemberInfo mi = sf.GetMethod();

				if( strSkipClassName != string.Empty && mi.DeclaringType.Name.IndexOf( strSkipClassName ) > -1 )
				{
					// don't include frames with this name
				}
				else
					sb.Append( StackFrameToString( sf ) );
			}
			sb.Append( Environment.NewLine );

			return sb.ToString();
		}
		#endregion

		/// <summary>
		/// turns a single stack frame object into an informative string
		/// </summary>
		/// <param name="sf"></param>
		/// <returns></returns>
		private static string StackFrameToString( StackFrame sf )
		{
			StringBuilder sb = new StringBuilder();
			int intParam;
			MemberInfo mi = sf.GetMethod();

			// build method name
			sb.Append( "	" );
			sb.Append( mi.DeclaringType.Namespace );
			sb.Append( "." );
			sb.Append( mi.DeclaringType.Name );
			sb.Append( "." );
			sb.Append( mi.Name );

			// build method params
			ParameterInfo[] objParameters = sf.GetMethod().GetParameters();

			sb.Append( "( " );
			intParam = 0;
			foreach( ParameterInfo objParameter in objParameters )
			{
				intParam += 1;
				if( intParam > 1 ) sb.Append( ", " );

				sb.Append( objParameter.ParameterType.Name );
				sb.Append( " " );
				sb.Append( objParameter.Name );
			}
			sb.Append( " )" );
			sb.Append( Environment.NewLine );

			// if source code is available, append location info
			sb.Append( "		" );
			if( string.IsNullOrEmpty( sf.GetFileName() ) )
			{
				sb.Append( System.IO.Path.GetFileName( _parentAssembly.CodeBase ) );
				// native code offset is always available
				sb.Append( ": N " );
				sb.Append( String.Format( "{0:#00000}", sf.GetNativeOffset() ) );
			}
			else
			{
				sb.Append( System.IO.Path.GetFileName( sf.GetFileName() ) );
				sb.Append( ": line " );
				sb.Append( String.Format( "{0:#0000}", sf.GetFileLineNumber() ) );
				sb.Append( ", col " );
				sb.Append( String.Format( "{0:#00}", sf.GetFileColumnNumber() ) );
				// if IL is available, append IL location info
				if( sf.GetILOffset() != System.Diagnostics.StackFrame.OFFSET_UNKNOWN )
				{
					sb.Append( ", IL " );
					sb.Append( String.Format( "{0:#0000}", sf.GetILOffset() ) );
				}
			}
			sb.Append( Environment.NewLine );

			return sb.ToString();
		}

		/// <summary>
		/// get IP address of this machine
		/// </summary>
		/// <remarks>Not an ideal method for a number of reasons (guess why!) but the alternatives are very ugly</remarks>
		/// <returns></returns>
		private static string GetCurrentIP()
		{
			try
			{
				return System.Net.Dns.GetHostEntry( System.Net.Dns.GetHostName() ).AddressList[0].ToString();
			}
			catch( Exception )
			{
				return "127.0.0.1";
			}
		}

		/// <summary>
		/// retrieve identity with fallback on error to safer method
		/// </summary>
		/// <returns></returns>
		private static string GetUserIdentity()
		{
			string identity = string.Empty;

			try
			{
				// exception-safe WindowsIdentity.GetCurrent retrieval returns "domain\username"
				// per MS, this sometimes randomly fails with "Access Denied" particularly on NT4
				identity = System.Security.Principal.WindowsIdentity.GetCurrent().Name;
			}
			catch( Exception )
			{
				try
				{
					// exception-safe "domain\username" retrieval from Environment
					identity = System.Environment.UserDomainName + "\\" + System.Environment.UserName;
				}
				catch( Exception )
				{
					identity = string.Empty;
				}
			}

			return identity;
		}

		private static DateTime GetAssemblyBuildDate( Assembly asm, bool forceFileDate )
		{
			Version ver = asm.GetName().Version;
			DateTime result;

			if( forceFileDate )
				result = GetAssemblyFileTime( asm );
			else
			{
				result = new DateTime( 2000, 1, 1 ).AddDays( ver.Build ).AddSeconds( ver.Revision * 2 );

				if( TimeZone.IsDaylightSavingTime( result, TimeZone.CurrentTimeZone.GetDaylightChanges( result.Year ) ) )
					result = result.AddHours( 1 );

				if( result > DateTime.Now || ver.Build < 730 || ver.Revision == 0 )
					result = GetAssemblyFileTime( asm );

			}

			return result;
		}

		//
		// exception-safe file attrib retrieval; we don't care if this fails
		//
		private static DateTime GetAssemblyFileTime( Assembly asm )
		{
			try
			{
				return System.IO.File.GetLastWriteTime( asm.Location );
			}
			catch( Exception )
			{
				return DateTime.MaxValue;
			}
		}
		#endregion
	}
}