// PathResolver.cs: Contributed by Ethan J. Brown (ethan_j_brown@hotmail.com)
// Used to turn a local style path into a UNC style and vice versa
// Will also resolve SUBST'd paths up the SUBST tree until the real drive letter is found

#region Copyright  2002-2004 The Genghis Group
/*
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not claim
 * that you wrote the original software. If you use this software in a product,
 * an acknowledgment in the product documentation is IsRequired, as shown here:
 * 
 * Portions Copyright  2002-2004 The Genghis Group (http://www.genghisgroup.com/).
 * 
 * 2. No substantial portion of the source code of this library may be redistributed
 * without the express written permission of the copyright holders, where
 * "substantial" is defined as enough code to be recognizably from this library. 
*/
#endregion
#region Features
/*
 * - Will resolve a UNC path as a local path if possible - this includes remote resources that have been
 *    mapped and local resources that have been shared
 * - Will resolve a local path to a UNC path if possible - this includes local resources that have been
 *    shared and remote resources that have been mapped
 * - Fully compatible with SUBST'd drives, and any variation of mapping/sharing/SUBST'ing
 * - Provides event notifications if an extremely long list of files is being resolved
 * - Caches map and share data to prevent constantly refreshing it at each method invocation
 * - PathList prevents insertion of paths that differ only by case
 * - PathList prevents setting on a path if the new path differs from an existing path by case
 * - ResolvedPathList provides the original path as a key, and a resolved equivalent as a value
 * - The resolved path is String.Empty when no resolution was possible
*/
#endregion
#region Limitations
/*
 *  - None that I'm aware of
*/
#endregion
#region History
/*
 * [7/12/2002]    First Release
 * 
*/
#endregion
#region TODO
/*
 *  - Add an asynchronous callback mechanism to begin resolving an extensive pathset
 *    and to notify client code when the operation has completed
 *  - Possibly make methods static
 *  - Maybe add a granularity to the event firing
 *  - Make sure that it's not necessary to use MatchingPathList within ResolvedPathList
 */
#endregion

using System;
using System.Collections;
using System.Runtime.InteropServices;
using Genghis.Win32;

namespace Genghis
{
    #region PathResolveTypeEnum enum
    /// <summary>
    /// An enumeration specifying what type of path to generate from the given path
    /// </summary>
    public enum PathResolveTypeEnum
    {
        /// <summary>
        /// Generate a UNC style path from the given path
        /// </summary>
        UNC = 0,
        /// <summary>
        /// Generate a local style path from the given path
        /// </summary>
        Local = 1
    }
    #endregion
    #region IPathResolver interface
    /// <summary>
    /// The interface that a class must support to be a PathResolver
    /// </summary>
    public interface IPathResolver
    {
        //TODO: make these methods static?
        /// <summary>
        /// Resolve a single path to the destination type specified by ResolutionType
        /// </summary>
        /// <param name="Path">A <see cref="string"/> representing the original path</param>
        /// <param name="ResolutionType">A <see cref="PathResolveTypeEnum"/> resolution type</param>
        /// <returns>The resolved path if a resolution was possible.  Otherwise, returns <see cref="string.Empty"/></returns>
        string ResolvePath(string Path, PathResolveTypeEnum ResolutionType);
        
        /// <summary>
        /// Resolve multiples path specified in a <see cref="PathList"/> to the destination type specified by ResolutionType
        /// </summary>
        /// <param name="PathList">A <see cref="PathList"/> containing a list of unique paths</param>
        /// <param name="ResolutionType">A <see cref="PathResolveTypeEnum"/> resolution type</param>
        /// <returns>A <see cref="IResolvedPathList"/> hashtable-style read-only collection of original paths as keys, and their corresponding resolutions as values.  
        /// A resolved value is <see cref="string.Empty"/> if no resolution was possible.
        /// </returns>
        IResolvedPathList ResolvePathList(PathList PathList, PathResolveTypeEnum ResolutionType);
    }
    #endregion

    /// <summary>
    /// The delegate corresponding to the <see cref="PathResolver.OnPathResolved"/> event
    /// </summary>
    public delegate void PathResolvedEventHandler(object sender, PathResolverEventArgs e);

    /// <summary>
    /// Class that contains data relevant to the <see cref="PathResolvedEventHandler"/> delegate
    /// </summary>
    /// <remarks>
    /// This class provides statistical information and other data about the last path resolution operation
    /// </remarks>
    public class PathResolverEventArgs : System.EventArgs 
    {  
        private readonly int currentIndex;
        private readonly int total;
        private readonly string path;
        private readonly string resolvedPath;     
        private readonly PathResolveTypeEnum pathResolveTypeEnum;

        /// <summary>
        /// The constructor used to create an instance of the PathResolverEventArgs class
        /// </summary>
        /// <param name="currentIndex">The index of the current item within the original <see cref="PathList"/> passed to the <see cref="PathResolver"/></param>
        /// <param name="total">The total number of paths in the original <see cref="PathList"/> passed to the <see cref="PathResolver"/></param>
        /// <param name="path">The current path within the original <see cref="PathList"/> passed to the <see cref="PathResolver"/></param>
        /// <param name="resolvedPath">The resolution of the current path within the original <see cref="PathList"/> passed to the <see cref="PathResolver"/></param>
        /// <param name="pathResolveTypeEnum">The <see cref="PathResolveTypeEnum"/> type of operation being performed on the current path</param>
        public PathResolverEventArgs(int currentIndex, int total, string path, string resolvedPath, PathResolveTypeEnum pathResolveTypeEnum)
        {
            this.currentIndex = currentIndex;
            this.total = total;
            this.path = path;
            this.resolvedPath = resolvedPath;
            this.pathResolveTypeEnum = pathResolveTypeEnum;
        }

        /// <summary>
        /// Returns the index within the original <see cref="PathList"/> passed to the <see cref="PathResolver"/>
        /// </summary>
        public int CurrentIndex
        {
            get { return this.currentIndex; }
        }

        /// <summary>
        /// Returns the total number of paths in the original <see cref="PathList"/> passed to the <see cref="PathResolver"/>
        /// </summary>
        public int Total
        {
            get { return this.total; }
        }

        /// <summary>
        /// Returns the current path within the original <see cref="PathList"/> passed to the <see cref="PathResolver"/>
        /// </summary>
        public string Path
        {
            get { return this.path; }
        }

        /// <summary>
        /// Returns the resolution of the current path within the original <see cref="PathList"/> passed to the <see cref="PathResolver"/>
        /// </summary>
        public string ResolvedPath
        {
            get { return this.resolvedPath; }
        }

        /// <summary>
        /// The <see cref="PathResolveTypeEnum"/> type of operation being performed on the current path
        /// </summary>
        public PathResolveTypeEnum ResolutionType
        {
            get { return this.pathResolveTypeEnum; }
        }
    }

    /// <summary>
    /// Class responsible for resolving single paths, or a list of paths via a <see cref="PathList"/>
    /// </summary>
    public class PathResolver : IPathResolver
    {
        //[DllImport("mpr.dll")]
        //private static extern int WNetGetConnection(string localName, System.Text.StringBuilder remoteName, ref int remoteNameLength);

        [DllImport("kernel32.dll")]
        private static extern int QueryDosDevice(string deviceName, System.Text.StringBuilder dosDeviceName, int dosDeviceNameLength);

        //private const int NO_ERROR = 0;
        private const int MAX_PATH = 260;
        private ShareMap    shareMap = null;
        private MappedDriveMap mappedDriveMap = null;
        private string computerName = System.Windows.Forms.SystemInformation.ComputerName;

        /// <summary>
        /// Occurrs any time a path has been resolved
        /// </summary>
        public event PathResolvedEventHandler PathResolved;

        /// <summary>
        /// Initializes a new instace of PathResolver
        /// </summary>
        /// <remarks>
        /// Allocates an internal instance of the <see cref="SystemShares.Shares"/> class, 
        /// in addition to an internal instance of the <see cref="MappedDrives.Maps"/> class
        /// which will be used for path resolution
        /// </remarks>
        public PathResolver()
        {
            SystemShares.SharesChanged += new SystemSharesSharesChangedEventHandler(this.shareMap_SharesChanged);
            shareMap = SystemShares.Shares;

            MappedDrives.MappedDrivesChanged += new MappedDrivesChangedEventHandler(this.mappedDriveMap_MappedDrivesChanged);
            mappedDriveMap = MappedDrives.Maps;
        }

        #region Implementation of IPathResolver
        /// <summary>
        /// Resolve a single path to the destination type specified by ResolutionType
        /// </summary>
        /// <param name="Path">A <see cref="string"/> representing the original path</param>
        /// <param name="ResolutionType">A <see cref="PathResolveTypeEnum"/> resolution type</param>
        /// <returns>The resolved path if a resolution was possible.  Otherwise, returns <see cref="string.Empty"/></returns>
        public string ResolvePath(string Path, PathResolveTypeEnum ResolutionType)
        {
            string resolvedPath = string.Empty;

            switch (ResolutionType)
            {
                case PathResolveTypeEnum.Local:
                    resolvedPath = resolvePathLocal(Path);
                    break;
                case PathResolveTypeEnum.UNC:
                    resolvedPath = resolvePathUNC(Path);
                    break;
            }
            OnPathResolved(this, new PathResolverEventArgs(0, 1, Path, resolvedPath, ResolutionType));
            return resolvedPath;
        }

        /// <summary>
        /// Resolve multiples path specified in a <see cref="PathList"/> to the destination type specified by ResolutionType
        /// </summary>
        /// <param name="PathList">A <see cref="PathList"/> containing a list of unique paths</param>
        /// <param name="ResolutionType">A <see cref="PathResolveTypeEnum"/> resolution type</param>
        /// <returns>A <see cref="IResolvedPathList"/> hashtable-style read-only collection of original paths as keys, and their corresponding resolutions as values.  
        /// A resolved value is <see cref="string.Empty"/> if no resolution was possible.
        /// </returns>
        public Genghis.IResolvedPathList ResolvePathList(PathList PathList, PathResolveTypeEnum ResolutionType)
        {
            int index = 0;
            string resolvedPath = string.Empty;
            if (null == PathList)
                return null;

            ResolvedPathList    resolvedList = new ResolvedPathList();

            foreach (string path in PathList)
            {
                switch (ResolutionType)
                {
                    case PathResolveTypeEnum.Local:
                        resolvedPath = resolvePathLocal(path);        
                        break;
                    case PathResolveTypeEnum.UNC:
                        resolvedPath = resolvePathUNC(path);
                        break;
                }
                
                ((IPrivateResolvedPathList)resolvedList).Add(path, resolvedPath);
                //TODO: fire an event here - at a more un-granular level?
                OnPathResolved(this, new PathResolverEventArgs(index, PathList.Count, path, resolvedPath, ResolutionType));

                ++index;
            }

            return resolvedList as IResolvedPathList;
        }
        #endregion

        /// <summary>
        /// Indicates that the collection of system-wide shares has been updated
        /// </summary>
        /// <param name="sender">A <see cref="object"/> representing the originator of the event</param>
        /// <param name="e">A <see cref="System.EventArgs"/> representing additional data relevant to the event</param>
        private void shareMap_SharesChanged(object sender, System.EventArgs e)
        {
            //lock the share collection and reload it
            lock (shareMap.SyncRoot)
            {
                shareMap = SystemShares.Shares;
            }
        }

        /// <summary>
        /// Indicates that the collection of system-wide mapped drives has been updated
        /// </summary>
        /// <param name="sender">A <see cref="object"/> representing the originator of the event</param>
        /// <param name="e">A <see cref="System.EventArgs"/> representing additional data relevant to the event</param>
        private void mappedDriveMap_MappedDrivesChanged(object sender, System.EventArgs e)
        {
            //lock the mapped drive collection and reload it
            lock (mappedDriveMap.SyncRoot)
            {
                mappedDriveMap = MappedDrives.Maps;
            }
        }

        /// <summary>
        /// Occurrs any time a path has been resolved
        /// </summary>
        /// <param name="sender">A <see cref="object"/> representing the originator of the event</param>
        /// <param name="e">A <see cref="System.EventArgs"/> representing additional data relevant to the event</param>
        protected void OnPathResolved(object sender, PathResolverEventArgs e)
        {
            //this prevents event subscribers from blowing up our event publishers
            try
            {
                if (this.PathResolved != null) 
                    // Invokes the delegates. 
                    this.PathResolved(sender, e);
            }
            catch (System.Exception)
            {}
        }


        private string resolvePathUNC(string localPath)
        {
            bool isDirectory = System.IO.Directory.Exists(localPath);
            bool isFile = System.IO.File.Exists(localPath); 
            System.Uri    pathURI = null;
            string driveLetter = string.Empty, UNCPath = string.Empty;
            //int    networkPathLength = 0;

            if (isFile || isDirectory)
            {
                try
                {
                    pathURI = new System.Uri(localPath);
                }
                //an exception is thrown if its an invalidly formatted path
                catch (System.Exception)
                {}

                //our work here is done - we were passed a valid UNC
                if ((null != pathURI) && pathURI.IsUnc)
                    return localPath;
                //it's some form of a local path - might be subst'd, mapped or shared
                else
                {
                    driveLetter = localPath.Substring(0, 2).ToUpper();
                    /*
                    //is it mapped?
                    //TODO - just check against our already built mapped drives collection?
                    System.Text.StringBuilder    networkPath = new System.Text.StringBuilder(MAX_PATH);
                    networkPathLength = networkPath.Capacity;
                    //we've got a mapped drive here - return the drives equivalent UNC
                    if (NO_ERROR == WNetGetConnection(driveLetter, networkPath, ref networkPathLength))
                        return networkPath.ToString() + localPath.Substring(2);
                    */

                    foreach (string local in mappedDriveMap.Keys)
                    {
                        if (0 == local.ToUpper().IndexOf(driveLetter))
                            return mappedDriveMap[local] + localPath.Substring(2);
                    }
                    
                    //key is the UNC path, value is the local path
                    foreach (string sharePath in shareMap.Values)
                    {
                        //we matched the shared path inside of the passed path
                        if (0 == localPath.ToUpper().IndexOf(sharePath.ToUpper()))
                        {
                            UNCPath = "\\\\" + computerName + "\\";
                            
                            foreach (string shareName in shareMap.Keys)
                                if (shareMap[shareName] == sharePath)
                                {
                                    UNCPath += shareName;
                                    break;
                                }        
                    
                            UNCPath += localPath.Substring(sharePath.Length);
                            break;
                        }
                    }

                    //we've built a valid UNC path from the shared path we were given
                    if ((isFile && System.IO.File.Exists(UNCPath)) || (isDirectory && System.IO.Directory.Exists(UNCPath)))
                        return UNCPath;

                    //last chance - hope for subst'd drives - check each drive along the way
                    return resolvePathUNC(resolvePathSUBST(localPath));
                }
            }

            return string.Empty;
        }

        private string resolvePathLocal(string UNCPath)
        {
            bool isDirectory = System.IO.Directory.Exists(UNCPath);
            bool isFile = System.IO.File.Exists(UNCPath); 
            System.Uri    pathURI = null;

            if (isFile || isDirectory)
            {
                try
                {
                    pathURI = new System.Uri(UNCPath);
                }
                catch (System.Exception)
                {}

                if ((null != pathURI) && pathURI.IsUnc)
                {
                    //we've matched our computer name - after 2 \'s
                    if (2 == UNCPath.ToUpper().IndexOf(computerName.ToUpper()))
                    {
                        //discard the 2 preceding and 1 trailing \
                        string filePath = UNCPath.Substring(computerName.Length + 3);
                        
                        //key is the UNC path, value is the local path
                        foreach (string shareName in shareMap.Keys)
                            if (0 == filePath.ToUpper().IndexOf(shareName.ToUpper()))
                                //we matched the sharename
                                return shareMap[shareName] + filePath.Substring(shareName.Length);
                    }
                    //maybe someone elses computer
                    else
                    {
                        //key is the local drive, value is the UNC share name
                        foreach (string localPath in mappedDriveMap.Keys)
                        {
                            if (0 == UNCPath.ToUpper().IndexOf(mappedDriveMap[localPath].ToUpper()))
                                return localPath + ((mappedDriveMap[localPath].Length < UNCPath.Length) ? UNCPath.Substring(mappedDriveMap[localPath].Length + 1) : "");
                        }
                    }
                }
                //non-UNC, local styled path, that we know exists
                else
                {
                    string resolvedSUBST = string.Empty;

                    //follow the SUBST'd drive letter backwards if possible
                    do 
                    {
                        resolvedSUBST = resolvePathSUBST(UNCPath);
                        if (string.Empty != resolvedSUBST)
                            UNCPath = resolvedSUBST;
                    }
                    while (resolvedSUBST != string.Empty);

                    return UNCPath;
                }
            }

            return string.Empty;
        }


        private string resolvePathSUBST(string SUBSTPath)
        {
            //last chance - check for SUBST'd drives
            System.Text.StringBuilder    dosDeviceName = new System.Text.StringBuilder(MAX_PATH);
            //we've got a mapped drive here - return the drives equivalent UNC
            if (0 != QueryDosDevice(SUBSTPath.Substring(0, 2), dosDeviceName, MAX_PATH))
            {
                string dosDevice = dosDeviceName.ToString();

                //we've got a SUBST'd drive letter mapping
                if (0 == dosDevice.IndexOf("\\??\\"))
                    //change the drive letter, and return
                    return dosDevice.Substring(4, 1) + SUBSTPath.Substring(1);
            }
            return string.Empty;
        }

    }


    /// <summary>
    ///        A strongly-typed collection of unique case-insensitive <see cref="System.String"/> as paths.
    ///        Originally generated by Chris Sells CollectionGen, modified for internal use, and for case insensitivity
    /// </summary>
    [Serializable]
    public class PathList : ICollection, IList, IEnumerable, ICloneable
    {
        #region Interfaces
        /// <summary>
        ///        Supports type-safe iteration over a <see cref="PathList"/>.
        /// </summary>
        public interface IPathListEnumerator
        {
            /// <summary>
            ///        Gets the current element in the collection.
            /// </summary>
            System.String Current {get;}

            /// <summary>
            ///        Advances the enumerator to the next element in the collection.
            /// </summary>
            /// <exception cref="InvalidOperationException">
            ///        The collection was modified after the enumerator was created.
            /// </exception>
            /// <returns>
            ///        <c>true</c> if the enumerator was successfully advanced to the next element; 
            ///        <c>false</c> if the enumerator has passed the end of the collection.
            /// </returns>
            bool MoveNext();

            /// <summary>
            ///        Sets the enumerator to its initial position, before the first element in the collection.
            /// </summary>
            void Reset();
        }
        #endregion

        private const int DEFAULT_CAPACITY = 16;

        #region Implementation (data)
        private System.String[] m_array;
        private int m_count = 0;
        private int m_version = 0;
        #endregion
    
        #region Static Wrappers
        /// <summary>
        ///        Creates a synchronized (thread-safe) wrapper for a 
        ///     <c>PathList</c> instance.
        /// </summary>
        /// <returns>
        ///     An <c>PathList</c> wrapper that is synchronized (thread-safe).
        /// </returns>
        public static PathList Synchronized(PathList list)
        {
            if(list==null)
                throw new ArgumentNullException("list");
            return new SyncPathList(list);
        }
        
        /// <summary>
        ///        Creates a read-only wrapper for a 
        ///     <c>PathList</c> instance.
        /// </summary>
        /// <returns>
        ///     An <c>PathList</c> wrapper that is read-only.
        /// </returns>
        public static PathList ReadOnly(PathList list)
        {
            if(list==null)
                throw new ArgumentNullException("list");
            return new ReadOnlyPathList(list);
        }
        #endregion

        #region Construction
        /// <summary>
        ///        Initializes a new instance of the <c>PathList</c> class
        ///        that is empty and has the default initial capacity.
        /// </summary>
        public PathList()
        {
            m_array = new System.String[DEFAULT_CAPACITY];
        }
        
        /// <summary>
        ///        Initializes a new instance of the <c>PathList</c> class
        ///        that has the specified initial capacity.
        /// </summary>
        /// <param name="capacity">
        ///        The number of elements that the new <c>PathList</c> is initially capable of storing.
        ///    </param>
        public PathList(int capacity)
        {
            m_array = new System.String[capacity];
        }

        /// <summary>
        ///        Initializes a new instance of the <c>PathList</c> class
        ///        that contains elements copied from the specified <c>PathList</c>.
        /// </summary>
        /// <param name="c">The <c>PathList</c> whose elements are copied to the new collection.</param>
        public PathList(PathList c)
        {
            m_array = new System.String[c.Count];
            AddRange(c);
        }

        /// <summary>
        ///        Initializes a new instance of the <c>PathList</c> class
        ///        that contains elements copied from the specified <see cref="System.String"/> array.
        /// </summary>
        /// <param name="a">The <see cref="System.String"/> array whose elements are copied to the new list.</param>
        public PathList(System.String[] a)
        {
            m_array = new System.String[a.Length];
            AddRange(a);
        }
        #endregion
        
        #region Operations (type-safe ICollection)
        /// <summary>
        ///        Gets the number of elements actually contained in the <c>PathList</c>.
        /// </summary>
        public virtual int Count
        {
            get { return m_count; }
        }

        /// <summary>
        ///        Copies the entire <c>PathList</c> to a one-dimensional
        ///        string array.
        /// </summary>
        /// <param name="array">The one-dimensional <see cref="System.String"/> array to copy to.</param>
        public virtual void CopyTo(System.String[] array)
        {
            this.CopyTo(array, 0);
        }

        /// <summary>
        ///        Copies the entire <c>PathList</c> to a one-dimensional
        ///        <see cref="System.String"/> array, starting at the specified index of the target array.
        /// </summary>
        /// <param name="array">The one-dimensional <see cref="System.String"/> array to copy to.</param>
        /// <param name="start">The zero-based index in <paramref name="array"/> at which copying begins.</param>
        public virtual void CopyTo(System.String[] array, int start)
        {
            if (m_count > array.GetUpperBound(0) + 1 - start)
                throw new System.ArgumentException("Destination array was not long enough.");
            
            Array.Copy(m_array, start, array, 0, m_count); 
        }

        /// <summary>
        ///        Gets a value indicating whether access to the collection is synchronized (thread-safe).
        /// </summary>
        /// <returns>true if access to the ICollection is synchronized (thread-safe); otherwise, false.</returns>
        public virtual bool IsSynchronized
        {
            get { return m_array.IsSynchronized; }
        }

        /// <summary>
        ///        Gets an object that can be used to synchronize access to the collection.
        /// </summary>
        public virtual object SyncRoot
        {
            get { return m_array.SyncRoot; }
        }
        #endregion
        
        #region Operations (type-safe IList)
        /// <summary>
        ///        Gets or sets the <see cref="System.String"/> at the specified index.
        /// </summary>
        /// <param name="index">The zero-based index of the element to get or set.</param>
        /// <exception cref="ArgumentOutOfRangeException">
        ///        <para><paramref name="index"/> is less than zero</para>
        ///        <para>-or-</para>
        ///        <para><paramref name="index"/> is equal to or greater than <see cref="PathList.Count"/>.</para>
        /// </exception>
        /// <remarks>
        /// Will not set the path at the current index with the passed in string value
        /// if that path is already found elsewhere in the collection.
        /// </remarks>
        public virtual System.String this[int index]
        {
            get
            {
                ValidateIndex(index); // throws
                return m_array[index]; 
            }
            set
            {
                ValidateIndex(index); // throws
                if (!this.Contains(value))
                {
                    ++m_version; 
                    m_array[index] = value; 
                }
            }
        }

        /// <summary>
        ///        Adds a <see cref="System.String"/> to the end of the <c>PathList</c>.
        /// </summary>
        /// <param name="item">The <see cref="System.String"/> to be added to the end of the <c>PathList</c>.</param>
        /// <returns>The index at which the value has been added.</returns>
        /// <remarks>
        /// Will not add item as a new path if that path is already found elsewhere in the collection.
        /// </remarks>
        public virtual int Add(System.String item)
        {
            if (!this.Contains(item))
            {
                if (m_count == m_array.Length)
                    EnsureCapacity(m_count + 1);

                m_array[m_count] = item;
                m_version++;
                ++m_count;
            }
            return m_count;
        }
        
        /// <summary>
        ///        Removes all elements from the <c>PathList</c>.
        /// </summary>
        public virtual void Clear()
        {
            ++m_version;
            m_array = new System.String[DEFAULT_CAPACITY];
            m_count = 0;
        }
        
        /// <summary>
        ///        Creates a shallow copy of the <see cref="PathList"/>.
        /// </summary>
        public virtual object Clone()
        {
            PathList newColl = new PathList(m_count);
            Array.Copy(m_array, 0, newColl.m_array, 0, m_count);
            newColl.m_count = m_count;
            newColl.m_version = m_version;

            return newColl;
        }

        /// <summary>
        ///        Determines whether a given <see cref="System.String"/> is in the <c>PathList</c>.
        /// </summary>
        /// <param name="item">The <see cref="System.String"/> to check for.</param>
        /// <returns><c>true</c> if <paramref name="item"/> is found in the <c>PathList</c>; otherwise, <c>false</c>.</returns>
        public virtual bool Contains(System.String item)
        {
            for (int i=0; i != m_count; ++i)
                if (m_array[i].ToUpper().Equals(item.ToUpper()))
                    return true;
            return false;
        }

        /// <summary>
        ///        Returns the zero-based index of the first occurrence of a <see cref="System.String"/>
        ///        in the <c>PathList</c>.
        /// </summary>
        /// <param name="item">The <see cref="System.String"/> to locate in the <c>PathList</c>.</param>
        /// <returns>
        ///        The zero-based index of the first occurrence of <paramref name="item"/> 
        ///        in the entire <c>PathList</c>, if found; otherwise, -1.
        ///    </returns>
        public virtual int IndexOf(System.String item)
        {
            for (int i=0; i != m_count; ++i)
                if (m_array[i].Equals(item))
                    return i;
            return -1;
        }

        /// <summary>
        ///        Inserts an element into the <c>PathList</c> at the specified index.
        /// </summary>
        /// <param name="index">The zero-based index at which <paramref name="item"/> should be inserted.</param>
        /// <param name="item">The <see cref="System.String"/> to insert.</param>
        /// <exception cref="ArgumentOutOfRangeException">
        ///        <para><paramref name="index"/> is less than zero</para>
        ///        <para>-or-</para>
        ///        <para><paramref name="index"/> is equal to or greater than <see cref="PathList.Count"/>.</para>
        /// </exception>
        /// <remarks>
        /// Will not add item as a new path if that path is already found elsewhere in the collection.
        /// </remarks>
        public virtual void Insert(int index, System.String item)
        {
            ValidateIndex(index, true); // throws
            
            if (!this.Contains(item))
            {
                if (m_count == m_array.Length)
                    EnsureCapacity(m_count + 1);

                if (index < m_count)
                {
                    Array.Copy(m_array, index, m_array, index + 1, m_count - index);
                }

                m_array[index] = item;
                m_count++;
                m_version++;
            }
        }

        /// <summary>
        ///        Removes the first occurrence of a specific <see cref="System.String"/> from the <c>PathList</c>.
        /// </summary>
        /// <param name="item">The <see cref="System.String"/> to remove from the <c>PathList</c>.</param>
        /// <exception cref="ArgumentException">
        ///        The specified <see cref="System.String"/> was not found in the <c>PathList</c>.
        /// </exception>
        public virtual void Remove(System.String item)
        {           
            int i = IndexOf(item);
            if (i < 0)
                throw new System.ArgumentException("Cannot remove the specified item because it was not found in the specified Collection.");
            
            ++m_version;
            RemoveAt(i);
        }

        /// <summary>
        ///        Removes the element at the specified index of the <c>PathList</c>.
        /// </summary>
        /// <param name="index">The zero-based index of the element to remove.</param>
        /// <exception cref="ArgumentOutOfRangeException">
        ///        <para><paramref name="index"/> is less than zero</para>
        ///        <para>-or-</para>
        ///        <para><paramref name="index"/> is equal to or greater than <see cref="PathList.Count"/>.</para>
        /// </exception>
        public virtual void RemoveAt(int index)
        {
            ValidateIndex(index); // throws
            
            m_count--;

            if (index < m_count)
            {
                Array.Copy(m_array, index + 1, m_array, index, m_count - index);
            }
            
            // We can't set the deleted entry equal to null, because it might be a value type.
            // Instead, we'll create an empty single-element array of the right type and copy it 
            // over the entry we want to erase.
            System.String[] temp = new System.String[1];
            Array.Copy(temp, 0, m_array, m_count, 1);
            m_version++;
        }

        /// <summary>
        ///        Gets a value indicating whether the collection has a fixed size.
        /// </summary>
        /// <value>true if the collection has a fixed size; otherwise, false. The default is false</value>
        public virtual bool IsFixedSize
        {
            get { return false; }
        }

        /// <summary>
        ///        gets a value indicating whether the IList is read-only.
        /// </summary>
        /// <value>true if the collection is read-only; otherwise, false. The default is false</value>
        public virtual bool IsReadOnly
        {
            get { return false; }
        }
        #endregion

        #region Operations (type-safe IEnumerable)
        
        /// <summary>
        ///        Returns an enumerator that can iterate through the <c>PathList</c>.
        /// </summary>
        /// <returns>An <see cref="Enumerator"/> for the entire <c>PathList</c>.</returns>
        public virtual IPathListEnumerator GetEnumerator()
        {
            return new Enumerator(this);
        }
        #endregion

        #region Public helpers (just to mimic some nice features of ArrayList)
        
        /// <summary>
        ///        Gets or sets the number of elements the <c>PathList</c> can contain.
        /// </summary>
        public virtual int Capacity
        {
            get { return m_array.Length; }
            
            set
            {
                if (value < m_count)
                    value = m_count;

                if (value != m_array.Length)
                {
                    if (value > 0)
                    {
                        System.String[] temp = new System.String[value];
                        Array.Copy(m_array, temp, m_count);
                        m_array = temp;
                    }
                    else
                    {
                        m_array = new System.String[DEFAULT_CAPACITY];
                    }
                }
            }
        }

        /// <summary>
        ///        Adds the elements of another <c>PathList</c> to the current <c>PathList</c>.
        /// </summary>
        /// <param name="x">The <c>PathList</c> whose elements should be added to the end of the current <c>PathList</c>.</param>
        /// <returns>The new <see cref="PathList.Count"/> of the <c>PathList</c>.</returns>
        public virtual int AddRange(PathList x)
        {
            if (m_count + x.Count >= m_array.Length)
                EnsureCapacity(m_count + x.Count);
            
            Array.Copy(x.m_array, 0, m_array, m_count, x.Count);
            m_count += x.Count;
            m_version++;

            return m_count;
        }

        /// <summary>
        ///        Adds the elements of a <see cref="System.String"/> array to the current <c>PathList</c>.
        /// </summary>
        /// <param name="x">The <see cref="System.String"/> array whose elements should be added to the end of the <c>PathList</c>.</param>
        /// <returns>The new <see cref="PathList.Count"/> of the <c>PathList</c>.</returns>
        /// <remarks>
        /// Checks each path string one at a time against the existing collection.  Will not add item as a new path if that path is already found elsewhere in the collection.
        /// </remarks>
        public virtual int AddRange(System.String[] x)
        {
            if (m_count + x.Length >= m_array.Length)
                EnsureCapacity(m_count + x.Length);

            foreach(string path in x)
            {
                if (!this.Contains(path))
                {
                    m_array[m_count] = path;
                    ++m_version;
                    ++m_count;
                }
            }

            return m_count;
        }
        
        /// <summary>
        ///        Sets the capacity to the actual number of elements.
        /// </summary>
        public virtual void TrimToSize()
        {
            this.Capacity = m_count;
        }

        #endregion

        #region Implementation (helpers)

        /// <exception cref="ArgumentOutOfRangeException">
        ///        <para><paramref name="index"/> is less than zero</para>
        ///        <para>-or-</para>
        ///        <para><paramref name="index"/> is equal to or greater than <see cref="PathList.Count"/>.</para>
        /// </exception>
        private void ValidateIndex(int i)
        {
            ValidateIndex(i, false);
        }

        /// <exception cref="ArgumentOutOfRangeException">
        ///        <para><paramref name="index"/> is less than zero</para>
        ///        <para>-or-</para>
        ///        <para><paramref name="index"/> is equal to or greater than <see cref="PathList.Count"/>.</para>
        /// </exception>
        private void ValidateIndex(int i, bool allowEqualEnd)
        {
            int max = (allowEqualEnd)?(m_count):(m_count-1);
            if (i < 0 || i > max)
                throw new System.ArgumentOutOfRangeException("Index was out of range.  Must be non-negative and less than the size of the collection.", (object)i, "Specified argument was out of the range of valid values.");
        }

        private void EnsureCapacity(int min)
        {
            int newCapacity = ((m_array.Length == 0) ? DEFAULT_CAPACITY : m_array.Length * 2);
            if (newCapacity < min)
                newCapacity = min;

            this.Capacity = newCapacity;
        }

        #endregion
        
        #region Implementation (ICollection)

        void ICollection.CopyTo(Array array, int start)
        {
            this.CopyTo((System.String[])array, start);
        }

        #endregion

        #region Implementation (IList)

        object IList.this[int i]
        {
            get { return (object)this[i]; }
            set { this[i] = (System.String)value; }
        }

        int IList.Add(object x)
        {
            return this.Add((System.String)x);
        }

        bool IList.Contains(object x)
        {
            return this.Contains((System.String)x);
        }

        int IList.IndexOf(object x)
        {
            return this.IndexOf((System.String)x);
        }

        void IList.Insert(int pos, object x)
        {
            this.Insert(pos, (System.String)x);
        }

        void IList.Remove(object x)
        {
            this.Remove((System.String)x);
        }

        void IList.RemoveAt(int pos)
        {
            this.RemoveAt(pos);
        }

        #endregion

        #region Implementation (IEnumerable)

        IEnumerator IEnumerable.GetEnumerator()
        {
            return (IEnumerator)(this.GetEnumerator());
        }

        #endregion

        #region Nested enumerator class
        /// <summary>
        ///        Supports simple iteration over a <see cref="PathList"/>.
        /// </summary>
        private class Enumerator : IEnumerator, IPathListEnumerator
        {
            #region Implementation (data)
            
            private PathList m_collection;
            private int m_index;
            private int m_version;
            
            #endregion
        
            #region Construction
            
            /// <summary>
            ///        Initializes a new instance of the <c>Enumerator</c> class.
            /// </summary>
            /// <param name="tc"></param>
            internal Enumerator(PathList tc)
            {
                m_collection = tc;
                m_index = -1;
                m_version = tc.m_version;
            }
            
            #endregion
    
            #region Operations (type-safe IEnumerator)
            
            /// <summary>
            ///        Gets the current element in the collection.
            /// </summary>
            public System.String Current
            {
                get { return m_collection[m_index]; }
            }

            /// <summary>
            ///        Advances the enumerator to the next element in the collection.
            /// </summary>
            /// <exception cref="InvalidOperationException">
            ///        The collection was modified after the enumerator was created.
            /// </exception>
            /// <returns>
            ///        <c>true</c> if the enumerator was successfully advanced to the next element; 
            ///        <c>false</c> if the enumerator has passed the end of the collection.
            /// </returns>
            public bool MoveNext()
            {
                if (m_version != m_collection.m_version)
                    throw new System.InvalidOperationException("Collection was modified; enumeration operation may not execute.");

                ++m_index;
                return (m_index < m_collection.Count) ? true : false;
            }

            /// <summary>
            ///        Sets the enumerator to its initial position, before the first element in the collection.
            /// </summary>
            public void Reset()
            {
                m_index = -1;
            }
            #endregion
    
            #region Implementation (IEnumerator)
            
            object IEnumerator.Current
            {
                get { return (object)(this.Current); }
            }
            
            #endregion
        }
        #endregion
        
        #region Nested Syncronized Wrapper class
        private class SyncPathList : PathList
        {
            #region Implementation (data)
            private PathList m_collection;
            private object m_root;
            #endregion

            #region Construction
            internal SyncPathList(PathList list)
            {
                m_root = list.SyncRoot;
                m_collection = list;
            }
            #endregion
            
            #region Type-safe ICollection
            public override void CopyTo(System.String[] array)
            {
                lock(this.m_root)
                    m_collection.CopyTo(array);
            }

            public override void CopyTo(System.String[] array, int start)
            {
                lock(this.m_root)
                    m_collection.CopyTo(array,start);
            }
            public override int Count
            {
                get
                { 
                    lock(this.m_root)
                        return m_collection.Count;
                }
            }

            public override bool IsSynchronized
            {
                get { return true; }
            }

            public override object SyncRoot
            {
                get { return this.m_root; }
            }
            #endregion
            
            #region Type-safe IList
            public override System.String this[int i]
            {
                get
                {
                    lock(this.m_root)
                        return m_collection[i];
                }
                set
                {
                    lock(this.m_root)
                        m_collection[i] = value; 
                }
            }

            public override int Add(System.String x)
            {
                lock(this.m_root)
                    return m_collection.Add(x);
            }
            
            public override void Clear()
            {
                lock(this.m_root)
                    m_collection.Clear();
            }

            public override bool Contains(System.String x)
            {
                lock(this.m_root)
                    return m_collection.Contains(x);
            }

            public override int IndexOf(System.String x)
            {
                lock(this.m_root)
                    return m_collection.IndexOf(x);
            }

            public override void Insert(int pos, System.String x)
            {
                lock(this.m_root)
                    m_collection.Insert(pos,x);
            }

            public override void Remove(System.String x)
            {           
                lock(this.m_root)
                    m_collection.Remove(x);
            }

            public override void RemoveAt(int pos)
            {
                lock(this.m_root)
                    m_collection.RemoveAt(pos);
            }
            
            public override bool IsFixedSize
            {
                get {return m_collection.IsFixedSize;}
            }

            public override bool IsReadOnly
            {
                get {return m_collection.IsReadOnly;}
            }
            #endregion

            #region Type-safe IEnumerable
            public override IPathListEnumerator GetEnumerator()
            {
                lock(m_root)
                    return m_collection.GetEnumerator();
            }
            #endregion

            #region Public Helpers
            // (just to mimic some nice features of ArrayList)
            public override int Capacity
            {
                get
                {
                    lock(this.m_root)
                        return m_collection.Capacity;
                }
                
                set
                {
                    lock(this.m_root)
                        m_collection.Capacity = value;
                }
            }

            public override int AddRange(PathList x)
            {
                lock(this.m_root)
                    return m_collection.AddRange(x);
            }

            public override int AddRange(System.String[] x)
            {
                lock(this.m_root)
                    return m_collection.AddRange(x);
            }
            #endregion
        }
        #endregion

        #region Nested Read Only Wrapper class
        private class ReadOnlyPathList : PathList
        {
            #region Implementation (data)
            private PathList m_collection;
            #endregion

            #region Construction
            internal ReadOnlyPathList(PathList list)
            {
                m_collection = list;
            }
            #endregion
            
            #region Type-safe ICollection
            public override void CopyTo(System.String[] array)
            {
                m_collection.CopyTo(array);
            }

            public override void CopyTo(System.String[] array, int start)
            {
                m_collection.CopyTo(array,start);
            }
            public override int Count
            {
                get {return m_collection.Count;}
            }

            public override bool IsSynchronized
            {
                get { return m_collection.IsSynchronized; }
            }

            public override object SyncRoot
            {
                get { return this.m_collection.SyncRoot; }
            }
            #endregion
            
            #region Type-safe IList
            public override System.String this[int i]
            {
                get { return m_collection[i]; }
                set { throw new NotSupportedException("This is a Read Only Collection and can not be modified"); }
            }

            public override int Add(System.String x)
            {
                throw new NotSupportedException("This is a Read Only Collection and can not be modified");
            }
            
            public override void Clear()
            {
                throw new NotSupportedException("This is a Read Only Collection and can not be modified");
            }

            public override bool Contains(System.String x)
            {
                return m_collection.Contains(x);
            }

            public override int IndexOf(System.String x)
            {
                return m_collection.IndexOf(x);
            }

            public override void Insert(int pos, System.String x)
            {
                throw new NotSupportedException("This is a Read Only Collection and can not be modified");
            }

            public override void Remove(System.String x)
            {           
                throw new NotSupportedException("This is a Read Only Collection and can not be modified");
            }

            public override void RemoveAt(int pos)
            {
                throw new NotSupportedException("This is a Read Only Collection and can not be modified");
            }
            
            public override bool IsFixedSize
            {
                get {return true;}
            }

            public override bool IsReadOnly
            {
                get {return true;}
            }
            #endregion

            #region Type-safe IEnumerable
            public override IPathListEnumerator GetEnumerator()
            {
                return m_collection.GetEnumerator();
            }
            #endregion

            #region Public Helpers
            // (just to mimic some nice features of ArrayList)
            public override int Capacity
            {
                get { return m_collection.Capacity; }
                
                set { throw new NotSupportedException("This is a Read Only Collection and can not be modified"); }
            }

            public override int AddRange(PathList x)
            {
                throw new NotSupportedException("This is a Read Only Collection and can not be modified");
            }

            public override int AddRange(System.String[] x)
            {
                throw new NotSupportedException("This is a Read Only Collection and can not be modified");
            }
            #endregion
        }
        #endregion
    }




    /// <summary>
    /// Describes the public interface for a read-only PathList collection
    /// </summary>
    public interface IResolvedPathList : IPublicDictionary
    {
        //MatchingPathList this[long index] { get; }
        string this[long index] { get; }
        //MatchingPathList this[string key] { get; }
        string this[string key] { get; }
        bool Contains(string key);
    }

    internal interface IPrivateResolvedPathList : IPrivateDictionary
    {
        void Remove(string key);
        new void Clear();
        void Add(string key, string value);
        //MatchingPathList this[string key] { set;}
        string this[string key] { set; }
    }

    #region IPublicDictionary interface
    public interface IPublicDictionary
    {
        #region Public Implementation of IDictionary
        System.Collections.IDictionaryEnumerator GetEnumerator();
        bool Contains(object key);
        bool IsReadOnly    { get; }
        object this[object key] { get; }
        System.Collections.ICollection Values { get; }
        System.Collections.ICollection Keys { get; }
        bool IsFixedSize { get; }
        #endregion    
    }
    #endregion
    #region IPrivateDictionary interface
    internal interface IPrivateDictionary
    {    
        #region internal Implementation of IDictionary
        void Remove(object key);
        void Clear();
        void Add(object key, object value);
        object this[object key] { set;}
        #endregion
    }
    #endregion


    /// <summary>
    /// A hashtable of a given path, and its resolved path, or string.Empty if it couldn't be resolved
    /// Originally generated by Chris Sells CollectionGen, then modified for internal scope
    /// </summary>
    public class ResolvedPathList : IResolvedPathList, IPrivateResolvedPathList, ICollection, IEnumerable, ICloneable
    {
        protected Hashtable innerHash;
        
        #region "Constructors"
        internal  ResolvedPathList()
        {
            innerHash = new Hashtable();
        }
        internal ResolvedPathList(ResolvedPathList original)
        {
            innerHash = new Hashtable (original.innerHash);
        }
        internal ResolvedPathList(IDictionary dictionary)
        {
            innerHash = new Hashtable (dictionary);
        }

        internal ResolvedPathList(int capacity)
        {
            innerHash = new Hashtable(capacity);
        }

        internal ResolvedPathList(IDictionary dictionary, float loadFactor)
        {
            innerHash = new Hashtable(dictionary, loadFactor);
        }

        internal ResolvedPathList(IHashCodeProvider codeProvider, IComparer comparer)
        {
            innerHash = new Hashtable (codeProvider, comparer);
        }

        internal ResolvedPathList(int capacity, int loadFactor)
        {
            innerHash = new Hashtable(capacity, loadFactor);
        }

        internal ResolvedPathList(IDictionary dictionary, IHashCodeProvider codeProvider, IComparer comparer)
        {
            innerHash = new Hashtable (dictionary, codeProvider, comparer);
        }
        
        internal ResolvedPathList(int capacity, IHashCodeProvider codeProvider, IComparer comparer)
        {
            innerHash = new Hashtable (capacity, codeProvider, comparer);
        }

        internal ResolvedPathList(IDictionary dictionary, float loadFactor, IHashCodeProvider codeProvider, IComparer comparer)
        {
            innerHash = new Hashtable (dictionary, loadFactor, codeProvider, comparer);
        }

        internal ResolvedPathList(int capacity, float loadFactor, IHashCodeProvider codeProvider, IComparer comparer)
        {
            innerHash = new Hashtable (capacity, loadFactor, codeProvider, comparer);
        }

        
#endregion

        #region Implementation of IDictionary
        public System.Collections.IDictionaryEnumerator GetEnumerator()
        {
            return new ResolvedPathListEnumerator(this);
        }
        IEnumerator IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        void IPrivateResolvedPathList.Remove(string key)
        {
            innerHash.Remove (key);
        }
        void IPrivateDictionary.Remove(object key)
        {
            innerHash.Remove ((string)key);
        }

        bool IResolvedPathList.Contains(string key)
        {
            return innerHash.Contains(key);
        }
        bool IPublicDictionary.Contains(object key)
        {
            return innerHash.Contains((string)key);
        }

        void IPrivateResolvedPathList.Clear()
        {
            innerHash.Clear();        
        }
        void IPrivateDictionary.Clear()
        {
            innerHash.Clear();
        }


        void IPrivateResolvedPathList.Add(string key, string value)
        {
            innerHash.Add(key, value);
            /*
            if (innerHash.ContainsKey(key))
            {
                ((MatchingPathList)innerHash[key]).Add(value);
            }
            else
            {
                MatchingPathList rop = new MatchingPathList();
                rop.Add(value);
                innerHash.Add(key, rop);            
            }
            */
        }
        void IPrivateDictionary.Add(object key, object value)
        {
            ((IPrivateResolvedPathList)this).Add ((string)key, (string)value);
        }

        bool IPublicDictionary.IsReadOnly
        {
            get
            {
                return innerHash.IsReadOnly;
            }
        }

        #region IResolvedPathList
        
        //MatchingPathList IResolvedPathList.this[long index]
        string IResolvedPathList.this[long index]
        {
            get
            {
                long l = 0;
                foreach (DictionaryEntry de in innerHash)
                {
                    if (index == l)
                        return (string)de.Value;
                    //return MatchingPathList.ReadOnly((MatchingPathList)de.Value);
                    ++l;
                }
                return null;
            }
        }
        #endregion


        //MatchingPathList IResolvedPathList.this[string key]
        string IResolvedPathList.this[string key]
        {
            get
            {
                return (string)innerHash[key];
                //return MatchingPathList.ReadOnly((MatchingPathList) innerHash[key]);
            }
        }

        string IPrivateResolvedPathList.this[string key]
            //MatchingPathList IPrivateResolvedPathList.this[string key]
        {
            set 
            {
                if (innerHash.ContainsKey(key))
                {
                    innerHash[key] = value;
                }
                else
                {
                    innerHash.Add(key, value);
                }
            }
        }
        
        object IPublicDictionary.this[object key]
        {
            get
            {
                //return MatchingPathList.ReadOnly((MatchingPathList)innerHash[key]);
                return (string)innerHash[(string)key];
            }
        }
        object IPrivateDictionary.this[object key]
        {
            set
            {
                innerHash[(string)key] = (string)value;
            }
        }
        
        //if it isn't then we have to make this internal
        public System.Collections.ICollection Values
        {
            get
            {
                return innerHash.Values;
            }
        }

        //if it isn't then we have to make this internal
        public System.Collections.ICollection Keys
        {
            get
            {
                return innerHash.Keys;
            }
        }

        bool IPublicDictionary.IsFixedSize
        {
            get
            {
                return innerHash.IsFixedSize;
            }
        }
        #endregion

        #region Implementation of ICollection
        public void CopyTo(System.Array array, int index)
        {
            innerHash.CopyTo (array, index);
        }

        public bool IsSynchronized
        {
            get
            {
                return innerHash.IsSynchronized;
            }
        }

        public int Count
        {
            get
            {
                return innerHash.Count;
            }
        }

        public object SyncRoot
        {
            get
            {
                return innerHash.SyncRoot;
            }
        }
        #endregion

        #region Implementation of ICloneable
        public ResolvedPathList Clone()
        {
            ResolvedPathList clone = new ResolvedPathList();
            clone.innerHash = (Hashtable) innerHash.Clone();
            
            return clone;
        }
        object ICloneable.Clone()
        {
            return Clone();
        }
        #endregion
        
        #region "HashTable Methods"
        public bool ContainsKey (string key)
        {
            return innerHash.ContainsKey(key);
        }
        public bool ContainsValue (string value)
        {
            return innerHash.ContainsValue(value);
        }
        public static ResolvedPathList Synchronized(ResolvedPathList nonSync)
        {
            ResolvedPathList sync = new ResolvedPathList();
            sync.innerHash = Hashtable.Synchronized(nonSync.innerHash);

            return sync;
        }
        #endregion

        internal Hashtable InnerHash
        {
            get
            {
                return innerHash;
            }
        }
    }
    
    public class ResolvedPathListEnumerator : IDictionaryEnumerator
    {
        private IDictionaryEnumerator innerEnumerator;
            
        internal ResolvedPathListEnumerator (ResolvedPathList enumerable)
        {
            innerEnumerator = enumerable.InnerHash.GetEnumerator();
        }

        #region Implementation of IDictionaryEnumerator
        public string Key
        {
            get
            {
                return (string)innerEnumerator.Key;
            }
        }
        object IDictionaryEnumerator.Key
        {
            get
            {
                return Key;
            }
        }


        public string Value
        {
            get
            {
                return (string)innerEnumerator.Value;
            }
        }
        object IDictionaryEnumerator.Value
        {
            get
            {
                return Value;
            }
        }

        public System.Collections.DictionaryEntry Entry
        {
            get
            {
                return innerEnumerator.Entry;
            }
        }

        #endregion

        #region Implementation of IEnumerator
        public void Reset()
        {
            innerEnumerator.Reset();
        }

        public bool MoveNext()
        {
            return innerEnumerator.MoveNext();
        }

        public object Current
        {
            get
            {
                return innerEnumerator.Current;
            }
        }
        #endregion
    }


}    