using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using CCPM.Data;

namespace CCPM.ProjectRelated
{
	/// <summary>
	/// This class provides management of several data items that
	/// are in a many-to-one relationship with Projects. Each data item
	/// is represented by a sub-menu item of the "Administration" top-level
	/// menu item. In the click event for each item three delegates are instantiated
	/// that provide data management methods for this class. Those delegates,
	/// the ordinal name of the data item's ID column and name colum and the
	/// window title are passed into this forms constructor.
	/// </summary>
	public class ManageBase : System.Windows.Forms.Form
	{
		#region Class Fields
		protected System.Windows.Forms.ListBox listBoxOne;
		private System.Windows.Forms.Splitter splitter1;
		protected System.Windows.Forms.TabControl tabControl1;
		protected System.Windows.Forms.TabPage tabPageEdit;
		protected System.Windows.Forms.Button btnSave;
		protected System.Windows.Forms.TabPage tabPageNew;
		protected System.Windows.Forms.Button btnSaveNew;
		protected System.Windows.Forms.TextBox txtEdit;
		protected System.Windows.Forms.TextBox txtNew;
		protected System.Windows.Forms.RadioButton rbActive;
		protected System.Windows.Forms.RadioButton rbInactive;
		protected Genghis.Windows.Forms.RequiredFieldValidator rfvEdit;
		protected Genghis.Windows.Forms.RequiredFieldValidator rfvNew;
        protected Genghis.Windows.Forms.RequiredFieldValidator rfvDelete;
		protected DataSet ds = null;
		protected PopulateMethod populateMethod;
		protected UpdateMethod updateMethod;
		protected InsertMethod insertMethod;
        protected DeleteMethod deleteMethod;
		protected string nameCol;
		protected string idCol;
		#endregion

		
		private System.ComponentModel.Container components = null;
        protected Button btnDelete;

		public delegate DataSet PopulateMethod();

		public delegate void InsertMethod(string name);

		public delegate void UpdateMethod(string name, bool isActive, int id);

        public delegate void DeleteMethod( int id);

        private bool _hasInActive = true;
        private bool _hasDelete = false;

		//Get the ListBox selected value
		protected int ListBoxValue
		{
			get{return (int)listBoxOne.SelectedValue;}
		}
		//Get the edit form status selection
		protected bool IsActive
		{
			get{return rbActive.Checked ? true : false;}
		}
		//Set the window title
		protected string WindowTitle
		{
			set{this.Text = value;}
		}
		//Default Constructor
		public ManageBase(){}
		//Constructor

        public ManageBase(PopulateMethod populateMethod, InsertMethod insertMethod, UpdateMethod updateMethod, string nameCol, string idCol, bool isActiveCol, string windowTitle)
        {
            InitializeComponent();
            _hasInActive = isActiveCol;
            if (!_hasInActive)
            {
                rbActive.Visible = false;
                rbInactive.Visible = false;
            }
            this.populateMethod = populateMethod;
            this.insertMethod = insertMethod;
            this.updateMethod = updateMethod;
            this.nameCol = nameCol;
            this.idCol = idCol;
            this.WindowTitle = windowTitle;

            this.PopulateList();
            this.SetEditControls(this.ListBoxValue);
        }

		public ManageBase(PopulateMethod populateMethod, InsertMethod insertMethod, UpdateMethod updateMethod, string nameCol, string idCol, string windowTitle)
		{
			InitializeComponent();
			this.populateMethod = populateMethod;
			this.insertMethod = insertMethod;
			this.updateMethod = updateMethod;
			this.nameCol = nameCol;
			this.idCol = idCol;
			this.WindowTitle = windowTitle;
			
			this.PopulateList();
			this.SetEditControls(this.ListBoxValue);
		}

        public ManageBase(PopulateMethod populateMethod, InsertMethod insertMethod, UpdateMethod updateMethod, DeleteMethod deleteMethod, string nameCol, string idCol, bool isActiveCol, bool isDeleteBtn, string windowTitle)
        {
            InitializeComponent();
            _hasInActive = isActiveCol;
            if (!_hasInActive)
            {
                rbActive.Visible = false;
                rbInactive.Visible = false;
            }

            _hasDelete = isDeleteBtn;
            if ( _hasDelete)
            {
                btnDelete.Visible = true;
            }

            this.populateMethod = populateMethod;
            this.insertMethod = insertMethod;
            this.updateMethod = updateMethod;
            this.deleteMethod = deleteMethod;
            this.nameCol = nameCol;
            this.idCol = idCol;
            this.WindowTitle = windowTitle;

            this.PopulateList();
            this.SetEditControls(this.ListBoxValue);
        }

		//validate input for update
		protected virtual bool ValidateEdit()
		{
			txtEdit.Focus();
			btnSave.Focus();
			
			return rfvEdit.IsValid;
		}
		//validate input for insert
		protected virtual bool ValidateNew()
		{
			txtNew.Focus();
			btnSaveNew.Focus();
			
			return rfvNew.IsValid;
		}
        //validate input for delete
        protected virtual bool ValidateDelete()
        {
            txtEdit.Focus();
            btnDelete.Focus();

            return rfvDelete.IsValid;
        }
		
		//set the list of items
		protected void PopulateList()
		{
			ds = this.populateMethod();
			listBoxOne.DataSource = ds.Tables[0];
			listBoxOne.DisplayMember = this.nameCol;
			listBoxOne.ValueMember = this.idCol;
		}

		//Select the ListBox item that matches the DataRowView item referenced by the ListBox 
		protected void SetListBoxSelection(string colName, int id)
		{
			for(int i = 0; i < listBoxOne.Items.Count; i ++)
			{
				if(id == (int)((DataRowView)listBoxOne.Items[i])[colName])
				{
					listBoxOne.SetSelected(i, true);
				}
			}
		}
		//Match the parameters passed in to a row in the DataTable and set the edit controls to the data in that row.
		protected virtual void SetEditControls(int id)
		{
			foreach(DataRow dr in ds.Tables[0].Rows)
			{
				if((int)dr[idCol] == id)
				{
					txtEdit.Text = (string)dr[nameCol];
                    if (_hasInActive)
                    {
                        switch ((bool)dr["IsActive"])
                        {
                            case true:
                                rbActive.Checked = true;
                                rbInactive.Checked = false;
                                break;
                            case false:
                                rbInactive.Checked = true;
                                rbActive.Checked = false;
                                break;
                        }
                    }
				}
			}
		}


		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}


		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(ManageBase));
            this.listBoxOne = new System.Windows.Forms.ListBox();
            this.splitter1 = new System.Windows.Forms.Splitter();
            this.tabControl1 = new System.Windows.Forms.TabControl();
            this.tabPageEdit = new System.Windows.Forms.TabPage();
            this.btnDelete = new System.Windows.Forms.Button();
            this.rbInactive = new System.Windows.Forms.RadioButton();
            this.rbActive = new System.Windows.Forms.RadioButton();
            this.btnSave = new System.Windows.Forms.Button();
            this.txtEdit = new System.Windows.Forms.TextBox();
            this.tabPageNew = new System.Windows.Forms.TabPage();
            this.btnSaveNew = new System.Windows.Forms.Button();
            this.txtNew = new System.Windows.Forms.TextBox();
            this.rfvEdit = new Genghis.Windows.Forms.RequiredFieldValidator();
            this.rfvNew = new Genghis.Windows.Forms.RequiredFieldValidator();
            this.rfvDelete = new Genghis.Windows.Forms.RequiredFieldValidator();
            this.tabControl1.SuspendLayout();
            this.tabPageEdit.SuspendLayout();
            this.tabPageNew.SuspendLayout();
            ((System.ComponentModel.ISupportInitialize)(this.rfvEdit)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.rfvNew)).BeginInit();
            ((System.ComponentModel.ISupportInitialize)(this.rfvDelete)).BeginInit();
            this.SuspendLayout();
            // 
            // listBoxOne
            // 
            this.listBoxOne.BorderStyle = System.Windows.Forms.BorderStyle.None;
            this.listBoxOne.ColumnWidth = 200;
            this.listBoxOne.Dock = System.Windows.Forms.DockStyle.Left;
            this.listBoxOne.Location = new System.Drawing.Point(0, 0);
            this.listBoxOne.Name = "listBoxOne";
            this.listBoxOne.Size = new System.Drawing.Size(160, 520);
            this.listBoxOne.TabIndex = 3;
            this.listBoxOne.MouseDown += new System.Windows.Forms.MouseEventHandler(this.listBoxOne_MouseDown);
            // 
            // splitter1
            // 
            this.splitter1.Location = new System.Drawing.Point(160, 0);
            this.splitter1.Name = "splitter1";
            this.splitter1.Size = new System.Drawing.Size(3, 524);
            this.splitter1.TabIndex = 4;
            this.splitter1.TabStop = false;
            // 
            // tabControl1
            // 
            this.tabControl1.Controls.Add(this.tabPageEdit);
            this.tabControl1.Controls.Add(this.tabPageNew);
            this.tabControl1.Dock = System.Windows.Forms.DockStyle.Fill;
            this.tabControl1.Location = new System.Drawing.Point(163, 0);
            this.tabControl1.Name = "tabControl1";
            this.tabControl1.SelectedIndex = 0;
            this.tabControl1.Size = new System.Drawing.Size(633, 524);
            this.tabControl1.TabIndex = 6;
            this.tabControl1.SelectedIndexChanged += new System.EventHandler(this.tabControl1_SelectedIndexChanged);
            // 
            // tabPageEdit
            // 
            this.tabPageEdit.Controls.Add(this.btnDelete);
            this.tabPageEdit.Controls.Add(this.rbInactive);
            this.tabPageEdit.Controls.Add(this.rbActive);
            this.tabPageEdit.Controls.Add(this.btnSave);
            this.tabPageEdit.Controls.Add(this.txtEdit);
            this.tabPageEdit.Location = new System.Drawing.Point(4, 22);
            this.tabPageEdit.Name = "tabPageEdit";
            this.tabPageEdit.Size = new System.Drawing.Size(625, 498);
            this.tabPageEdit.TabIndex = 0;
            this.tabPageEdit.Text = "Edit";
            this.tabPageEdit.Click += new System.EventHandler(this.tabPageEdit_Click);
            // 
            // btnDelete
            // 
            this.btnDelete.Location = new System.Drawing.Point(136, 88);
            this.btnDelete.Name = "btnDelete";
            this.btnDelete.Size = new System.Drawing.Size(75, 23);
            this.btnDelete.TabIndex = 4;
            this.btnDelete.Text = "Delete";
            this.btnDelete.Visible = false;
            this.btnDelete.Click += new System.EventHandler(this.btnDelete_Click);
            // 
            // rbInactive
            // 
            this.rbInactive.Location = new System.Drawing.Point(136, 56);
            this.rbInactive.Name = "rbInactive";
            this.rbInactive.Size = new System.Drawing.Size(104, 16);
            this.rbInactive.TabIndex = 3;
            this.rbInactive.Text = "Inactive";
            // 
            // rbActive
            // 
            this.rbActive.Location = new System.Drawing.Point(10, 56);
            this.rbActive.Name = "rbActive";
            this.rbActive.Size = new System.Drawing.Size(104, 16);
            this.rbActive.TabIndex = 2;
            this.rbActive.Text = "Active";
            // 
            // btnSave
            // 
            this.btnSave.Location = new System.Drawing.Point(8, 88);
            this.btnSave.Name = "btnSave";
            this.btnSave.Size = new System.Drawing.Size(75, 23);
            this.btnSave.TabIndex = 1;
            this.btnSave.Text = "Save";
            this.btnSave.Click += new System.EventHandler(this.SaveUpdate_Click);
            // 
            // txtEdit
            // 
            this.txtEdit.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.txtEdit.Location = new System.Drawing.Point(8, 16);
            this.txtEdit.Name = "txtEdit";
            this.txtEdit.Size = new System.Drawing.Size(596, 20);
            this.txtEdit.TabIndex = 0;
            // 
            // tabPageNew
            // 
            this.tabPageNew.Controls.Add(this.btnSaveNew);
            this.tabPageNew.Controls.Add(this.txtNew);
            this.tabPageNew.Location = new System.Drawing.Point(4, 22);
            this.tabPageNew.Name = "tabPageNew";
            this.tabPageNew.Size = new System.Drawing.Size(625, 498);
            this.tabPageNew.TabIndex = 1;
            this.tabPageNew.Text = "New";
            this.tabPageNew.Visible = false;
            // 
            // btnSaveNew
            // 
            this.btnSaveNew.Location = new System.Drawing.Point(8, 56);
            this.btnSaveNew.Name = "btnSaveNew";
            this.btnSaveNew.Size = new System.Drawing.Size(75, 23);
            this.btnSaveNew.TabIndex = 2;
            this.btnSaveNew.Text = "Save";
            this.btnSaveNew.Click += new System.EventHandler(this.btnSaveNew_Click);
            // 
            // txtNew
            // 
            this.txtNew.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.txtNew.Location = new System.Drawing.Point(8, 16);
            this.txtNew.Name = "txtNew";
            this.txtNew.Size = new System.Drawing.Size(596, 20);
            this.txtNew.TabIndex = 1;
            // 
            // rfvEdit
            // 
            this.rfvEdit.ControlToValidate = this.txtEdit;
            this.rfvEdit.ErrorMessage = "Entry may not be empty";
            this.rfvEdit.Icon = ((System.Drawing.Icon)(resources.GetObject("rfvEdit.Icon")));
            // 
            // rfvNew
            // 
            this.rfvNew.ControlToValidate = this.txtNew;
            this.rfvNew.ErrorMessage = "Entry may not be empty";
            this.rfvNew.Icon = ((System.Drawing.Icon)(resources.GetObject("rfvNew.Icon")));
            // 
            // rfvDelete
            // 
            this.rfvDelete.ControlToValidate = this.txtEdit;
            this.rfvDelete.ErrorMessage = "Entry may not be empty";
            this.rfvDelete.Icon = ((System.Drawing.Icon)(resources.GetObject("rfvDelete.Icon")));
            // 
            // ManageBase
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(796, 524);
            this.Controls.Add(this.tabControl1);
            this.Controls.Add(this.splitter1);
            this.Controls.Add(this.listBoxOne);
            this.Name = "ManageBase";
            this.Text = "Manage Base";
            this.tabControl1.ResumeLayout(false);
            this.tabPageEdit.ResumeLayout(false);
            this.tabPageEdit.PerformLayout();
            this.tabPageNew.ResumeLayout(false);
            this.tabPageNew.PerformLayout();
            ((System.ComponentModel.ISupportInitialize)(this.rfvEdit)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.rfvNew)).EndInit();
            ((System.ComponentModel.ISupportInitialize)(this.rfvDelete)).EndInit();
            this.ResumeLayout(false);

		}
		#endregion

		private void listBoxOne_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
		{
			this.SetEditControls(this.ListBoxValue);
		}


		protected void SaveUpdate_Click(object sender, EventArgs e)
		{
			if(ValidateEdit())
			{
				//Need to get the selected value before it is changed by PopulateList()
				int id = this.ListBoxValue; 
			
				this.updateMethod(txtEdit.Text, this.IsActive, id);

				this.PopulateList();

				this.SetEditControls(id);

				this.SetListBoxSelection(idCol, id);

			}
			else
			{
				txtEdit.Focus();
			}
		}


		private void btnSaveNew_Click(object sender, System.EventArgs e)
		{
			if(ValidateNew())
			{
				this.insertMethod(txtNew.Text);
				this.PopulateList();
				listBoxOne.SelectedIndex = listBoxOne.FindStringExact(txtNew.Text);
				txtNew.Text = string.Empty;
			}
			else
			{
				txtNew.Focus();
			}
		}

        private void btnDelete_Click(object sender, EventArgs e)
        {

            if (ValidateDelete())
            {
                //Need to get the selected value before it is changed by PopulateList()
                int id = this.ListBoxValue;

                this.deleteMethod(id);

                this.PopulateList();

                this.SetEditControls(this.ListBoxValue); 
                

            }
            else
            {
                txtEdit.Focus();
            }
        }

		//The concept is to set the TextBox on the edit form to the same item that was just entered on the new form.
		//public event tabControl1 TabIndexChanged;
        // MAR 2005-01-31 - added 
    	private void tabControl1_SelectedIndexChanged(object sender, System.EventArgs e)
		{
			if (tabControl1.SelectedTab.Text=="Edit")
			{
				this.txtEdit.Text = Convert.ToString(this.listBoxOne.Text);
			}
		}

        private void tabPageEdit_Click(object sender, EventArgs e)
        {

        }

 
		

	}
}
