using System;
using System.Drawing;
using ReportPrinting;
using System.IO;
using System.Collections;
using System.Data;

namespace CCPM.Report
{
	/// <summary>
	/// Configure the One Page Report with options specified by the user.
	/// MODIFICATIONS:
	/// MAR 2005-02-03 - add Assembly, State and Congressional District info
	/// MAR 2005-02-03 - tweak titles for optional plural  ie:  Programs to  Program(s)
	/// </summary>
	public class OnePageReportMaker : IReportMaker
	{
		
		OnePageReportOptions options = null;
		public OnePageReportOptions Options
		{
			get{return options;}
			set{options = value;}
		}
		
		public OnePageReportMaker(OnePageReportOptions options)
		{
			Options = options;
		}
		
		public void MakeDocument(ReportDocument reportDocument)
		{
			TextStyle.ResetStyles();

			// Create a ReportBuilder object that assists with building a report
			ReportBuilder builder = new ReportBuilder(reportDocument);
			//ReportSection section = new ReportSection();
			builder.StartLayeredLayout(true, true);

			TextStyle.Heading1.Bold = true;
			TextStyle.Heading1.Brush = Brushes.White;
			TextStyle.Heading1.SizeDelta = 8;
			TextStyle.Heading1.VerticalAlignment = VerticalAlignment.Middle;
		
			TextStyle.Heading2.Bold = true;
			TextStyle.Heading2.SizeDelta = 3;
			TextStyle.Heading2.MarginBottom = .1f;

			TextStyle.Heading3.Brush = Brushes.White;
			TextStyle.Heading3.MarginNear = .25f;
			TextStyle.Heading3.VerticalAlignment = VerticalAlignment.Middle;

			System.Drawing.SolidBrush blueBrush = new System.Drawing.SolidBrush(ColorTranslator.FromHtml("#314A84"));
			System.Drawing.SolidBrush tanBrush = new System.Drawing.SolidBrush(ColorTranslator.FromHtml("#E0BF98"));

			//Blue header box
			SectionBox titleBox = new SectionBox();
			titleBox.Background = blueBrush;
			titleBox.WidthPercent = 100f;
			titleBox.Height = 1.25f;
			titleBox.Padding = 0.1f;
			builder.AddSection (titleBox);
			
			//Project name box
			SectionBox nameBox = new SectionBox();
			nameBox.Background = blueBrush;
			nameBox.WidthPercent = 75;
			nameBox.MaxHeight = titleBox.Height;
			nameBox.Padding = 0.1f;
			nameBox.VerticalAlignment = VerticalAlignment.Middle;
			SectionText name = new SectionText(Options.ProjectName, TextStyle.Heading1);
			name.VerticalAlignment = VerticalAlignment.Middle;
			nameBox.AddSection(name);
			builder.AddSection(nameBox);
			
			//SCC Logo box
			SectionImage logoSection;
			System.Reflection.Assembly loadedAssembly = System.Reflection.Assembly.GetExecutingAssembly();
			Stream stream = loadedAssembly.GetManifestResourceStream(this.GetType(), "sccLogo.gif");
			Image img = Image.FromStream(stream);
			logoSection = new SectionImage(img);
			SectionBox logoBox = new SectionBox();
			logoBox.Width = 1.75f;
			logoBox.Height = 1.25f;
			logoBox.PaddingLeft = .1f;
			logoBox.PaddingTop = .05f;
			logoBox.Border = new Pen(blueBrush, .02f);
			logoBox.Background = Brushes.White;
			logoBox.HorizontalAlignment = HorizontalAlignment.Right;
			logoBox.AddSection(logoSection);
			builder.AddSection(logoBox);

			//Left column box
			SectionBox leftColumnBox = new SectionBox();
			leftColumnBox.Background = tanBrush;
			leftColumnBox.OffsetTop = 1.25f;
			leftColumnBox.Width = 2f;
			leftColumnBox.HeightPercent = 100f;
			builder.AddSection(leftColumnBox);

			//Left column items
			//Project programs
			LinearSections columnItems = new LinearSections();
			if(Options.ProjectPrograms != null)
			{
				SectionText section = new SectionText("Conservancy Program(s)", TextStyle.BoldStyle);
				section.MarginLeft = .1f;
				section.MarginTop = .1f;
				columnItems.AddSection(section);
				foreach(DataRow dr in Options.ProjectPrograms.Rows)
				{
					SectionText sectionSub = new SectionText((string)dr["ProgramType"], TextStyle.Normal);
					sectionSub.MarginLeft = .2f;
					columnItems.AddSection(sectionSub);
				}

				//Spacer
				columnItems.AddSection(new SectionText(string.Empty, TextStyle.Normal));
			}
			//Project types
			if(Options.ProjectTypes != null)
			{
				SectionText section = new SectionText("Project Type(s)", TextStyle.BoldStyle);
				section.MarginLeft = .1f;
				section.MarginTop = .1f;
				columnItems.AddSection(section);
				foreach(DataRow dr in Options.ProjectTypes.Rows)
				{
					SectionText sectionSub = new SectionText((string)dr["ProjectType"], TextStyle.Normal);
					sectionSub.MarginLeft = .2f;
					columnItems.AddSection(sectionSub);
				}

				//Spacer
				columnItems.AddSection(new SectionText(string.Empty, TextStyle.Normal));
			}
			//Project cities
			if(Options.Cities != null)
			{
				SectionText section = new SectionText("City(s)", TextStyle.BoldStyle);
				section.MarginLeft = .1f;
				section.MarginTop = .1f;
				columnItems.AddSection(section);
				foreach(DataRow dr in Options.Cities.Rows)
				{
					SectionText sectionSub = new SectionText((string)dr["CityName"], TextStyle.Normal);
					sectionSub.MarginLeft = .2f;
					columnItems.AddSection(sectionSub);
				}

				//Spacer
				columnItems.AddSection(new SectionText(string.Empty, TextStyle.Normal));
			}
			//Project county
			if(Options.Counties != null)
			{
				SectionText section = new SectionText("County(s)", TextStyle.BoldStyle);
				section.MarginLeft = .1f;
				section.MarginTop = .1f;
				columnItems.AddSection(section);
				foreach(DataRow dr in Options.Counties.Rows)
				{
					SectionText sectionSub = new SectionText((string)dr["CountyName"], TextStyle.Normal);
					sectionSub.MarginLeft = .2f;
					columnItems.AddSection(sectionSub);
				}

				//Spacer
				columnItems.AddSection(new SectionText(string.Empty, TextStyle.Normal));
			}
			//Project regions
			if(Options.Regions != null)
			{
				SectionText section = new SectionText("Region(s)", TextStyle.BoldStyle);
				section.MarginLeft = .1f;
				section.MarginTop = .1f;
				columnItems.AddSection(section);
				foreach(DataRow dr in Options.Regions.Rows)
				{
					SectionText sectionSub = new SectionText((string)dr["Region"], TextStyle.Normal);
					sectionSub.MarginLeft = .2f;
					columnItems.AddSection(sectionSub);
				}

				//Spacer
				columnItems.AddSection(new SectionText(string.Empty, TextStyle.Normal));
			}
			//Project size (Acres)
			if(Options.ProjectSize != -1)
			{
				SectionText section = new SectionText("Land Size", TextStyle.BoldStyle);
				section.MarginLeft = .1f;
				section.MarginTop = .1f;
				columnItems.AddSection(section);
				
				SectionText sectionSub = new SectionText(string.Concat(StripZero(Replace(Options.ProjectSize.ToString(),".00","")), " Acres"), TextStyle.Normal);
				sectionSub.MarginLeft = .2f;
				columnItems.AddSection(sectionSub);

				//Spacer
				columnItems.AddSection(new SectionText(string.Empty, TextStyle.Normal));
			}
			//Trail Miles
			if(Options.TrailMiles != -1)
			{
				SectionText section = new SectionText("Trail Miles", TextStyle.BoldStyle);
				section.MarginLeft = .1f;
				section.MarginTop = .1f;
				columnItems.AddSection(section);
				
				SectionText sectionSub = new SectionText(string.Concat(StripZero(Replace(Options.TrailMiles.ToString(),".00","")), " Miles"), TextStyle.Normal);
				sectionSub.MarginLeft = .2f;
				columnItems.AddSection(sectionSub);

				//Spacer
				columnItems.AddSection(new SectionText(string.Empty, TextStyle.Normal));
			}
			//Stream Miles
			if(Options.StreamMiles != -1)
			{
				SectionText section = new SectionText("Stream Miles", TextStyle.BoldStyle);
				section.MarginLeft = .1f;
				section.MarginTop = .1f;
				columnItems.AddSection(section);
				
				SectionText sectionSub = new SectionText(string.Concat(StripZero(Replace(Options.StreamMiles.ToString(),".00","")), " Miles"), TextStyle.Normal);
				sectionSub.MarginLeft = .2f;
				columnItems.AddSection(sectionSub);

				//Spacer
				columnItems.AddSection(new SectionText(string.Empty, TextStyle.Normal));
			}
			//Conservancy Funds
			if(Options.ConservancyFunds != -1)
			{
				SectionText section = new SectionText("Conservancy Contribution", TextStyle.BoldStyle);
				section.MarginLeft = .1f;
				section.MarginTop = .1f;
				columnItems.AddSection(section);
				//MAR 2005-01-27 - change to zero decimals
				SectionText sectionSub = new SectionText(Options.ConservancyFunds.ToString("C0"), TextStyle.Normal);
				sectionSub.MarginLeft = .2f;
				columnItems.AddSection(sectionSub);

				//Spacer
				columnItems.AddSection(new SectionText(string.Empty, TextStyle.Normal));
			}
			//Matching Funds
			if(Options.MatchingFunds != -1)
			{
				SectionText section = new SectionText("Matching Funds", TextStyle.BoldStyle);
				section.MarginLeft = .1f;
				section.MarginTop = .1f;
				columnItems.AddSection(section);
				
				//MAR 2005-01-27 - change to zero decimals
				SectionText sectionSub = new SectionText(Options.MatchingFunds.ToString("C0"), TextStyle.Normal);
				sectionSub.MarginLeft = .2f;
				columnItems.AddSection(sectionSub);

				//Spacer
				columnItems.AddSection(new SectionText(string.Empty, TextStyle.Normal));
			}
			//Total Funds
			if(Options.TotalFunds != -1)
			{
				SectionText section = new SectionText("Total Project Cost", TextStyle.BoldStyle);
				section.MarginLeft = .1f;
				section.MarginTop = .1f;
				columnItems.AddSection(section);
				
				//MAR 2005-01-27 - change to zero decimals
				SectionText sectionSub = new SectionText(Options.TotalFunds.ToString("C0"), TextStyle.Normal);
				sectionSub.MarginLeft = .2f;
				columnItems.AddSection(sectionSub);

				//Spacer
				columnItems.AddSection(new SectionText(string.Empty, TextStyle.Normal));
			}
			//Partners
			if(options.Partners.Count > 0)
			{
				SectionText section = new SectionText("Partner(s)", TextStyle.BoldStyle);
				section.MarginLeft = .1f;
				section.MarginTop = .1f;
				columnItems.AddSection(section);

				foreach(object partner in options.Partners)
				{
					SectionText sectionSub = new SectionText((string)partner, TextStyle.Normal);
					sectionSub.MarginLeft = .2f;
					columnItems.AddSection(sectionSub);
				}
				//Spacer
				columnItems.AddSection(new SectionText(string.Empty, TextStyle.Normal));
			}

			//MAR 2005-02-04 - added district sections
			//Assembly Districts
			if(options.AssemblyDistricts.Length>0)
			{
				SectionText section = new SectionText("Assembly District(s)", TextStyle.BoldStyle);
				section.MarginLeft = .1f;
				section.MarginTop = .1f;
				columnItems.AddSection(section);

				SectionText sectionSub = new SectionText(Options.AssemblyDistricts, TextStyle.Normal);
				sectionSub.MarginLeft = .2f;
				columnItems.AddSection(sectionSub);
			}
			//State Senate Districts
			if(options.StateSenateDistricts.Length>0)
			{
				SectionText section = new SectionText("State Senate District(s)", TextStyle.BoldStyle);
				section.MarginLeft = .1f;
				section.MarginTop = .1f;
				columnItems.AddSection(section);

				//MAR 2005-01-27 - change to zero decimals
				SectionText sectionSub = new SectionText(Options.StateSenateDistricts, TextStyle.Normal);
				sectionSub.MarginLeft = .2f;
				columnItems.AddSection(sectionSub);
			}
			//Congressional Districts
			if(options.CongressionalDistricts.Length>0)
			{
				SectionText section = new SectionText("Congressional District(s)", TextStyle.BoldStyle);
				section.MarginLeft = .1f;
				section.MarginTop = .1f;
				columnItems.AddSection(section);

				SectionText sectionSub = new SectionText(Options.CongressionalDistricts, TextStyle.Normal);
				sectionSub.MarginLeft = .2f;
				columnItems.AddSection(sectionSub);
			}


			leftColumnBox.AddSection(columnItems);

			//Left column footer
			SectionBox footerBox = new SectionBox();
			footerBox.Background = tanBrush;
			footerBox.Height = 1.75f;
			footerBox.MaxWidth = 2f;
			footerBox.UseFullWidth = true;
			footerBox.HorizontalAlignment = HorizontalAlignment.Center;
			footerBox.VerticalAlignment = VerticalAlignment.Bottom;

			LinearSections colFooter = new LinearSections();
			colFooter.SkipAmount = .2f;
			//SectionText footerSection1 = new SectionText("CALIFORNIA COASTAL CONSERVANCY",TextStyle.BoldStyle);
            SectionText footerSection1 = new SectionText(Options.AgencyLine1, TextStyle.BoldStyle);
            footerSection1.HorizontalAlignment = HorizontalAlignment.Center;
			colFooter.AddSection(footerSection1);

			//SectionText footerSection2 = new SectionText("1330 Broadway, 13th Floor \nOakland, CA 94612", TextStyle.Normal);
            SectionText footerSection2 = new SectionText(options.AgencyLine2, TextStyle.Normal);
            footerSection2.HorizontalAlignment = HorizontalAlignment.Center;
			colFooter.AddSection(footerSection2);

			//SectionText footerSection3 = new SectionText("Phone: 510-286-1015 \nFax: 510-286-0470 \nwww.scc.ca.gov", TextStyle.Normal);
            SectionText footerSection3 = new SectionText(Options.AgencyLine3, TextStyle.Normal);
            footerSection3.HorizontalAlignment = HorizontalAlignment.Center;
			colFooter.AddSection(footerSection3);

			footerBox.AddSection(colFooter);
			builder.AddSection(footerBox);


			//Description column
			SectionBox contentBox = new SectionBox();
			contentBox.OffsetTop = logoBox.Height;
			contentBox.OffsetLeft = leftColumnBox.Width;
			contentBox.HeightPercent = 100;
			contentBox.WidthPercent = 100;
			contentBox.BorderRight = new Pen(blueBrush, .02f);
			builder.AddSection(contentBox);

			//Add description content
			LinearSections content = new LinearSections();
			content.AddSection (new SectionText("Description", TextStyle.Heading2));
			content.AddSection(new SectionText(Options.Description, TextStyle.Normal));
			content.Margin = .25f;
			contentBox.AddSection(content);

			//Image
			if(Options.ProjectImage != null)
			{
				Bitmap bitmap = new Bitmap(Options.ProjectImage);
				float pixelWidth = bitmap.Width;
				float inchWidth = pixelWidth / 100f;
				float rightGutter = .25f; //white space between image and blue vertical line.
				float totalWidth = 7.5f;
		
				SectionImage imageSection = new SectionImage(bitmap);
				//imageSection.
				imageSection.PreserveAspectRatio = true;
				imageSection.VerticalAlignment = VerticalAlignment.Bottom;
				
				imageSection.MarginLeft = leftColumnBox.Width + .25f;
				//imageSection.MarginRight = .25f;
				imageSection.MarginRight = ((totalWidth - rightGutter) - imageSection.MarginLeft) - inchWidth;
				imageSection.MarginBottom = .55f;
				builder.AddSection(imageSection);

				if(Options.ImageCredit != null)
				{
					// MAR 2005-07-26 - would like to add Image Credit under photo, but this proved
					//                  to be not straight-forward. 

//					SectionBox ImageCreditSection = new SectionBox();
//					ImageCreditSection.OffsetTop = imageSection.MarginBottom;
//					ImageCreditSection.OffsetLeft = leftColumnBox.Width;
//					//ImageCreditSection.HeightPercent = 100;
//					//ImageCreditSection.WidthPercent = 100;
//					//ImageCreditSection.BorderRight = new Pen(blueBrush, .02f);
//					builder.AddSection(ImageCreditSection);
//					ImageCreditSection.AddSection (new SectionText("Photo by: ", TextStyle.Heading2));
//					ImageCreditSection.AddSection(new SectionText(Options.ImageCredit, TextStyle.Normal));
//					ImageCreditSection.Margin = .25f;
//					ImageCreditSection.AddSection(content);
				}

			
			}

			//Add bottom box (blue horizontal rectangle)
			SectionBox bottomBox = new SectionBox();
			bottomBox.Background = blueBrush;
			bottomBox.OffsetLeft = leftColumnBox.Width;
			bottomBox.WidthPercent = 100f;
			bottomBox.Height = .3f;
			bottomBox.VerticalAlignment = VerticalAlignment.Bottom;
			builder.AddSection(bottomBox);
			
			//Add staff
			string names = string.Empty;
			if(Options.Staff != null)
			{
				foreach(DataRow dr in Options.Staff.Rows)
				{
					names += ", ";
					names += (string)dr["Name"];
				}
				if(names != string.Empty)
				{
					names = names.Remove(0, 2);
				}
//				SectionBox staffBox = new SectionBox();
//				staffBox.MarginLeft = .25f;
//				staffBox.Height = .3f;
//				staffBox.WidthPercent = 100;
//				staffBox.VerticalAlignment = VerticalAlignment.Middle;
//				staffBox.AddSection(new SectionText(string.Concat("Conservancy Staff: ", names), TextStyle.Heading3));
//				bottomBox.AddSection(staffBox);
				bottomBox.AddSection(new SectionText(string.Concat("Conservancy Staff: ", names), TextStyle.Heading3));
			}

			builder.FinishLayeredLayout();
		}
		public static String Replace(String strText,String strFind,String strReplace)
		{
			int iPos=strText.IndexOf(strFind);
			String strReturn="";
			while(iPos!=-1)
			{
				strReturn+=strText.Substring(0,iPos) + strReplace;
				strText=strText.Substring(iPos+strFind.Length);
				iPos=strText.IndexOf(strFind);
			}
			if(strText.Length>0)
				strReturn+=strText;
			return strReturn;
		}
		public static String StripZero(String strText)
		{
			String strReturn=strText;
			int strLen = strText.Length;
			if(strText.IndexOf(".")>0)
			{
				// strip the trailing zero, if there is one
				if (strText.Substring(strLen-1)=="0")
				{
					strReturn = strText.Substring(0,strLen-1);
				}
			}
			return strReturn;
		}
	}

	/// <summary>
	/// Encapsulates the One Page Report selected options and information
	/// </summary>
	public class OnePageReportOptions
	{
		private string projectName;
		private DataTable projectPrograms = null;
		private DataTable projectTypes = null;
		private DataTable cities = null;
		private DataTable counties = null;
		private DataTable regions = null;
		private DataTable staff = null;
		private decimal projectSize = -1;
		private decimal trailMiles = -1;
		private decimal streamMiles = -1;
		private decimal conservancyFunds = -1;
		private decimal matchingFunds = -1;
		private decimal totalFunds = -1;
		private string description = string.Empty;
		private Image projectImage;
		
		// MAR 2005-07-22 - added
		private string imageCredit = string.Empty;

		private ArrayList partners = new ArrayList();

		// MAR 2004-02-04 - add district info
		private string assemblyDistricts=string.Empty;
		private string stateSenateDistricts=string.Empty;
		private string congressionalDistricts=string.Empty;

        private string agencyLine1 = string.Empty;
        private string agencyLine2 = string.Empty;
        private string agencyLine3 = string.Empty;

        public string AgencyLine1
        {
            get { return agencyLine1; }
            set { agencyLine1 = value; }
        }
        public string AgencyLine2
        {
            get { return agencyLine2; }
            set { agencyLine2 = value; }
        }
        public string AgencyLine3
        {
            get { return agencyLine3; }
            set { agencyLine3 = value; }
        }
        
        public string ProjectName
		{
			get{return projectName;}
			set{projectName = value;}
		}
		public DataTable ProjectPrograms
		{
			get{return projectPrograms;}
			set{projectPrograms = value;}
		}
		public DataTable ProjectTypes
		{
			get{return projectTypes;}
			set{projectTypes = value;}
		}
		public DataTable Cities
		{
			get{return cities;}
			set{cities = value;}
		}
		public DataTable Counties
		{
			get{return counties;}
			set{counties = value;}
		}
		public DataTable Regions
		{
			get{return regions;}
			set{regions = value;}
		}
		public DataTable Staff
		{
			get{return staff;}
			set{staff = value;}
		}

        public decimal ProjectSize
		{
			get{return projectSize;}
			set{projectSize = value;}
		}
		public decimal TrailMiles
		{
			get{return trailMiles;}
			set{trailMiles = value;}
		}
		public decimal StreamMiles
		{
			get{return streamMiles;}
			set{streamMiles = value;}
		}
		public decimal ConservancyFunds
		{
			get{return conservancyFunds;}
			set{conservancyFunds = value;}
		}
		public decimal MatchingFunds
		{
			get{return matchingFunds;}
			set{matchingFunds = value;}
		}
		public decimal TotalFunds
		{
			get{return totalFunds;}
			set{totalFunds = value;}
		}
		public string Description
		{
			get{return description;}
			set{description = value;}
		}
		public Image ProjectImage
		{
			get{return projectImage;}
			set{projectImage = value;}
		}

		public string ImageCredit
		{
			get{return imageCredit;}
			set{imageCredit = value;}
		}

		public ArrayList Partners
		{
			get{return partners;}
			set{partners = value;}
		}

		// MAR 2004-02-04 - add district info
		public string AssemblyDistricts
		{
			get{return assemblyDistricts;}
			set{assemblyDistricts = value;}
		}
		public string StateSenateDistricts
		{
			get{return stateSenateDistricts;}
			set{stateSenateDistricts = value;}
		}
		public string CongressionalDistricts
		{
			get{return congressionalDistricts;}
			set{congressionalDistricts = value;}
		}

		public OnePageReportOptions(){}
	}



			
}

